#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from collections import Iterable, OrderedDict
from xml.etree.ElementTree import HTML_EMPTY
import html as html_
from html.parser import HTMLParser
from typing import Union, Tuple, Callable
from weakref import WeakSet, WeakValueDictionary

from wdom.interface import NodeList, Event
from wdom.css import CSSStyleDeclaration
from wdom.node import Node, ParentNode, NonDocumentTypeChildNode, ChildNode
from wdom.node import DocumentFragment, Comment
from wdom.event import EventTarget
from wdom.webif import WebIF


class DOMTokenList:
    """List of DOM token.

    DOM token is a string which does not contain spases.
    """
    def __init__(self, owner, *args):
        self._list = list()
        self._owner = owner
        self._append(args)

    def __len__(self) -> int:
        return len(self._list)

    def __contains__(self, item: str) -> bool:
        return item in self._list

    def __iter__(self) -> str:
        for token in self._list:
            yield token

    def _validate_token(self, token: str):
        if not isinstance(token, str):
            raise TypeError(
                'Token must be str, but {} passed.'.format(type(token)))
        if ' ' in token:
            raise ValueError(
                'Token contains space characters, which are invalid.')

    def _append(self, token):
        if isinstance(token, str):
            for t in token.split(' '):
                self.add(t)
        elif isinstance(token, Iterable):
            for t in token:
                self._append(t)
        elif token is None:
            pass
        else:
            raise TypeError

    @property
    def length(self) -> int:
        """Number of DOM token in this list."""
        return len(self)

    def add(self, *tokens: Tuple[str]):
        """Add new tokens to list."""
        _new_tokens = []
        for token in tokens:
            self._validate_token(token)
            if token and token not in self:
                self._list.append(token)
                _new_tokens.append(token)
        if isinstance(self._owner, WebIF) and _new_tokens:
            self._owner.js_exec('addClass', _new_tokens)

    def remove(self, *tokens: Tuple[str]):
        """Remove tokens from list."""
        _removed_tokens = []
        for token in tokens:
            self._validate_token(token)
            if token in self:
                self._list.remove(token)
                _removed_tokens.append(token)
        if isinstance(self._owner, WebIF) and _removed_tokens:
            self._owner.js_exec('removeClass', _removed_tokens)

    def toggle(self, token: str):
        """Add or remove token to/from list.

        If token is in this list, the token will be removed. Otherwise add it
        to list.
        """
        self._validate_token(token)
        if token in self:
            self.remove(token)
        else:
            self.add(token)

    def item(self, index: int) -> str:
        """Return the token of the ``index``.

        ``index`` must be 0 or positive integer. If index is out of range,
        return None.
        """
        if 0 <= index < len(self):
            return self._list[index]
        else:
            return None

    def contains(self, token: str) -> bool:
        """Return if the token is in the list or not."""
        self._validate_token(token)
        return token in self

    def toString(self) -> str:
        """Return string representation of this list.

        Actually it will be a spase-separated tokens.
        """
        return ' '.join(self)


class Attr:
    """Attribute node.

    In the latest DOM specification, Attr interface does not inherits ``Node``
    interface. (Previously, Attr inherited Node interface.)
    """
    def __init__(self, name: str, value=None, owner: Node = None):
        self._name = name.lower()
        self._value = value
        self._owner = owner

    @property
    def html(self) -> str:
        """Return string representation of this.

        Used in start tag of HTML representation of the Element node.
        """
        if self._owner and self.name in self._owner._special_attr_boolean:
            return self.name
        else:
            if isinstance(self.value, str):
                value = html_.escape(self.value)
            else:
                value = self.value
            return '{name}="{value}"'.format(name=self.name, value=value)

    @property
    def name(self) -> str:
        """Name of this attr."""
        return self._name

    @property
    def value(self) -> str:
        """Value of this attr."""
        return self._value

    @value.setter
    def value(self, val):
        self._value = val

    @property
    def isId(self) -> bool:
        """Return True if this Attr is an ID node (name is ``id``)."""
        return self.name.lower() == 'id'


class NamedNodeMap:
    def __init__(self, owner):
        self._owner = owner
        self._dict = OrderedDict()

    def __len__(self) -> int:
        return len(self._dict)

    def __contains__(self, item: str) -> bool:
        return item in self._dict

    def __getitem__(self, index: Union[int, str]) -> Attr:
        if isinstance(index, int):
            return tuple(self._dict.values())[index]
        else:
            return None

    def __iter__(self) -> Attr:
        for attr in self._dict.keys():
            yield attr

    @property
    def length(self) -> int:
        return len(self)

    def getNamedItem(self, name: str) -> Attr:
        return self._dict.get(name, None)

    def setNamedItem(self, item: Attr):
        if not isinstance(item, Attr):
            raise TypeError('item must be an instance of Attr')
        if isinstance(self._owner, WebIF):
            self._owner.js_exec('setAttribute', item.name, item.value)
        self._dict[item.name] = item
        item._owner = self._owner

    def removeNamedItem(self, item: Attr) -> Attr:
        if not isinstance(item, Attr):
            raise TypeError('item must be an instance of Attr')
        if isinstance(self._owner, WebIF):
            self._owner.js_exec('removeAttribute', item.name)
        removed_item = self._dict.pop(item.name, None)
        if removed_item:
            removed_item._owner = self._owner
        return removed_item

    def item(self, index: int) -> Attr:
        if 0 <= index < len(self):
            return self._dict[tuple(self._dict.keys())[index]]
        else:
            return None

    def toString(self) -> str:
        return ' '.join(attr.html for attr in self._dict.values())


def _create_element(tag: str, name: str = None, base: type = None,
                    attr: dict = None):
    from wdom.tag import Tag
    from wdom.window import customElements
    if attr is None:
        attr = {}
    if name:
        base_class = customElements.get((name, tag))
    else:
        base_class = customElements.get((tag, None))
    if base_class is None:
        attr['_registered'] = False
        base_class = base or HTMLElement
    if issubclass(base_class, Tag):
        return base_class(**attr)
    else:
        return base_class(tag, **attr)


class Parser(HTMLParser):
    def __init__(self, *args, default_class=None, **kwargs):
        # Import here
        self.default_class = default_class or HTMLElement
        super().__init__(*args, **kwargs)
        self.elm = DocumentFragment()
        self.root = self.elm

    def handle_starttag(self, tag, attr):
        attrs = dict(attr)
        params = dict(parent=self.elm, **attrs)
        elm = _create_element(tag, attrs.get('is'), self.default_class, params)
        if self.elm:
            self.elm.append(elm)
        if tag not in HTML_EMPTY:
            self.elm = elm

    def handle_endtag(self, tag):
        self.elm = self.elm.parentNode

    def handle_data(self, data):
        if data and self.elm:
            self.elm.append(data)

    def handle_comment(self, comment: str):
        self.elm.append(Comment(comment))


_str_attr_doc = '''
Getter: Get value of ``{attr}`` attribute of this element, as string. If
`{attr}` is not defined, return empty string.
Setter: Set the value of
``{attr}`` attribute of this element.
Deleter: Remove ``{attr}`` attribute from this element.
'''

_bool_attr_doc = '''
Getter: Return True if this element has ``{attr}`` attribute. Otherwise return
False.
Setter: If True, add ``{attr}`` attribute to this element. Otherwise remove
``{attr}``.
Deleter: Remove ``{attr}`` attribute from this element.
'''


def _string_properties(attr) -> property:
    def getter(self) -> str:
        return self.getAttribute(attr) or ''

    def setter(self, value: str):
        self.setAttribute(attr, str(value))

    def deleter(self):
        self.removeAttribute(attr)

    return property(getter, setter, deleter, _str_attr_doc.format(attr=attr))


def _boolean_properties(attr) -> property:
    def getter(self: Node) -> bool:
        return bool(self.getAttribute(attr))

    def setter(self: Node, value: bool):
        if value:
            self.setAttribute(attr, True)
        else:
            self.removeAttribute(attr)

    def deleter(self):
        self.removeAttribute(attr)

    return property(getter, setter, deleter, _bool_attr_doc.format(attr=attr))


class ElementMeta(type):
    def __new__(cls, name, bases, namespace, **kwargs):
        for attr in namespace.get('_special_attr_string', []):
            namespace[attr] = _string_properties(attr)
        for attr in namespace.get('_special_attr_boolean', []):
            namespace[attr] = _boolean_properties(attr)
        new_cls = super().__new__(cls, name, bases, dict(namespace))
        return new_cls


class Element(Node, EventTarget, ParentNode, NonDocumentTypeChildNode,
              ChildNode, metaclass=ElementMeta):
    nodeType = Node.ELEMENT_NODE
    nodeValue = None
    _parser_default_class = None
    _elements = WeakSet()
    _elements_with_id = WeakValueDictionary()
    _should_escape_text = True
    _special_attr_string = ['id']
    _special_attr_boolean = []

    def __init__(self, tag: str='', parent: Node = None,
                 _registered: bool = True, **kwargs):
        self._registered = _registered
        self.tag = tag
        self._elements.add(self)  # used to suport custom elements
        self.attributes = NamedNodeMap(self)
        self.classList = DOMTokenList(self)
        super().__init__(parent=parent)

        if 'class_' in kwargs:
            kwargs['class'] = kwargs.pop('class_')
        if 'is_' in kwargs:
            kwargs['is'] = kwargs.pop('is_')
        for k, v in kwargs.items():
            self.setAttribute(k, v)

    def __copy__(self) -> 'Element':
        clone = type(self)(self.tag)
        for attr in self.attributes:
            clone.setAttribute(attr, self.getAttribute(attr))
        return clone

    def _get_attrs_by_string(self) -> str:
        # attrs = ' '.join(attr.html for attr in self.attributes.values())
        attrs = self.attributes.toString()
        classes = self.getAttribute('class')
        if classes:
            attrs = ' '.join((attrs.strip(), 'class="{}"'.format(classes)))
        return attrs.strip()

    @property
    def start_tag(self) -> str:
        tag = '<' + self.tag
        attrs = self._get_attrs_by_string()
        if attrs:
            tag = ' '.join((tag, attrs))
        return tag + '>'

    def _parse_html(self, html: str) -> DocumentFragment:
        parser = Parser(default_class=self._parser_default_class)
        parser.feed(html)
        return parser.root

    def _get_inner_html(self) -> str:
        return ''.join(child.html for child in self._children)

    def _set_inner_html(self, html: str):
        self._empty()
        self._append_child(self._parse_html(html))

    @property
    def innerHTML(self) -> str:
        return self._get_inner_html()

    @innerHTML.setter
    def innerHTML(self, html: str):
        self._set_inner_html(html)

    @property
    def end_tag(self) -> str:
        return '</{}>'.format(self.tag)

    @property
    def html(self) -> str:
        return self.start_tag + self.innerHTML + self.end_tag

    def insertAdjacentHTML(self, position: str, html: str):
        '''Parse ``html`` to DOM and insert to ``position``. ``position`` is a
        case-insensive string, and must be one of "beforeBegin", "afterBegin",
        "beforeEnd", or "afterEnd".
        '''
        df = self._parse_html(html)
        pos = position.lower()
        if pos == 'beforebegin':
            self.before(df)
        elif pos == 'afterbegin':
            self.prepend(df)
        elif pos == 'beforeend':
            self.append(df)
        elif pos == 'afterend':
            self.after(df)
        else:
            raise ValueError(
                'The value provided ({}) is not one of "beforeBegin", '
                '"afterBegin", "beforeEnd", or "afterEnd".'.format(position)
            )

    @property
    def outerHTML(self) -> str:
        return self.html

    @property
    def nodeName(self) -> str:
        return self.tag.upper()

    @property
    def tagName(self) -> str:
        return self.tag.upper()

    @property
    def localName(self) -> str:
        return self.tag.lower()

    def getAttribute(self, attr: str) -> str:
        if attr == 'class':
            if self.classList:
                return self.classList.toString()
            else:
                return None
        attr_node = self.getAttributeNode(attr)
        if attr_node is None:
            return None
        else:
            return attr_node.value

    def getAttributeNode(self, attr: str) -> Attr:
        return self.attributes.getNamedItem(attr)

    def hasAttribute(self, attr: str) -> bool:
        if attr == 'class':
            return bool(self.classList)
        else:
            return attr in self.attributes

    def hasAttributes(self) -> bool:
        return bool(self.attributes) or bool(self.classList)

    def _set_attribute(self, attr: str, value: Union[str, bool]):
        if attr == 'class':
            self.classList = DOMTokenList(self, value)
        else:
            if attr == 'id':
                if 'id' in self.attributes:
                    # remove old reference to self
                    self._elements_with_id.pop(self.id, None)
                # register this elements with new id
                self._elements_with_id[value] = self
            new_attr_node = Attr(attr, value)
            self.setAttributeNode(new_attr_node)

    def setAttribute(self, attr: str, value: Union[str, bool]):
        self._set_attribute(attr, value)

    def setAttributeNode(self, attr: Attr):
        self.attributes.setNamedItem(attr)

    def _remove_attribute(self, attr: str):
        if attr == 'class':
            self.classList = DOMTokenList(self)
        else:
            if attr == 'id':
                self._elements_with_id.pop(self.id, None)
            _attr = self.getAttributeNode(attr)
            if _attr:
                self.attributes.removeNamedItem(_attr)

    def removeAttribute(self, attr: str):
        self._remove_attribute(attr)

    def removeAttributeNode(self, attr: Attr) -> Attr:
        return self.attributes.removeNamedItem(attr)

    def getElementsBy(self, cond: Callable[['Element'], bool]) -> NodeList:
        '''Return list of child nodes which matches ``cond``.
        ``cond`` must be a function which gets a single argument ``node``,
        and returns bool. If the node matches requested condition, ``cond``
        should return True.
        This searches all child nodes recursively.
        '''
        elements = []
        for child in self.children:
            if cond(child):
                elements.append(child)
            elements.extend(child.getElementsBy(cond))
        return NodeList(elements)

    def getElementsByTagName(self, tag: str):
        _tag = tag.upper()
        return self.getElementsBy(lambda n: getattr(n, 'tagName') == _tag)

    def getElementsByClassName(self, class_name: str):
        return self.getElementsBy(
            lambda node: class_name in getattr(node, 'classList'))


class HTMLElement(Element):
    _special_attr_string = ['title', 'type']
    _special_attr_boolean = ['draggable', 'hidden']

    def __init__(self, *args, style: str=None, **kwargs):
        super().__init__(*args, **kwargs)
        self._style = CSSStyleDeclaration(style, owner=self)

    def _get_attrs_by_string(self) -> str:
        attrs = super()._get_attrs_by_string()
        style = self.getAttribute('style')
        if style:
            attrs += ' style="{}"'.format(style)
        return attrs.strip()

    def __copy__(self) -> 'HTMLElement':
        clone = super().__copy__()
        clone.style.update(self.style)
        return clone

    @property
    def end_tag(self) -> str:
        if self.tag in HTML_EMPTY:
            return ''
        else:
            return super().end_tag

    @property
    def style(self) -> CSSStyleDeclaration:
        return self._style

    @style.setter
    def style(self, style: str):
        if isinstance(style, str):
            self._style._parse_str(style)
        elif style is None:
            self._style._parse_str('')
        elif isinstance(style, CSSStyleDeclaration):
            self._style._owner = None
            style._owner = self
            self._style = style
            self._style.update()
        else:
            raise TypeError('Invalid type for style: {}'.format(type(style)))

    def getAttribute(self, attr: str) -> str:
        if attr == 'style':
            # if style is neither None nor empty, return None
            # otherwise, return style.cssText
            if self.style:
                return self.style.cssText
            else:
                return None
        else:
            return super().getAttribute(attr)

    def _set_attribute(self, attr: str, value: str):
        if attr == 'style':
            self.style = value
        else:
            super()._set_attribute(attr, value)

    def _remove_attribute(self, attr: str):
        if attr == 'style':
            self.style = None
        else:
            super()._remove_attribute(attr)


class FormControlMixin:
    def __init__(self, *args, form=None, **kwargs):
        self._form = None
        super().__init__(*args, **kwargs)
        from wdom.document import getElementById
        if isinstance(form, (str, int)):
            form = getElementById(form)
        if isinstance(form, HTMLFormElement):
            self._form = form
        elif form is not None:
            raise TypeError(
                '"form" attribute must be an HTMLFormElement or id of'
                'HTMLFormElement in the same document.'
            )

    @property
    def form(self) -> HTMLElement:
        if self._form:
            return self._form
        else:
            parent = self.parentNode
            while parent:
                if isinstance(parent, HTMLFormElement):
                    return parent
                else:
                    parent = parent.parentNode
            return None


class HTMLAnchorElement(HTMLElement):
    _special_attr_string = ['href', 'name', 'rel', 'src', 'target']


class HTMLButtonElement(HTMLElement):
    _special_attr_string = ['name', 'value']
    _special_attr_boolean = ['disabled']


class HTMLFormElement(HTMLElement):
    _special_attr_string = ['name']


class HTMLIFrameElement(HTMLElement):
    _special_attr_string = ['height', 'name', 'src', 'target', 'width']


class HTMLInputElement(FormControlMixin, HTMLElement):
    _special_attr_string = ['height', 'name', 'src', 'value', 'width']
    _special_attr_boolean = ['checked', 'disabled', 'multiple', 'readonly',
                             'required']

    def on_event_pre(self, e: Event):
        super().on_event_pre(e)
        if e.type in ('input', 'change'):
            # Update user inputs
            if self.type.lower() == 'checkbox':
                self._set_attribute('checked', e.currentTarget.get('checked'))
            elif self.type.lower() == 'radio':
                self._set_attribute('checked', e.currentTarget.get('checked'))
                for other in self._radio_group:
                    if other is not self:
                        other._remove_attribute('checked')
            else:
                self._set_attribute('value', e.currentTarget.get('value'))

    @property
    def defaultChecked(self) -> bool:
        return bool(self.getAttribute('defaultChecked'))

    @defaultChecked.setter
    def defaultChecked(self, value: bool):
        if value:
            self.setAttribute('defaultChecked', True)
            self.checked = True
        else:
            self.removeAttribute('defaultChecked')
            self.checked = False

    @property
    def defaultValue(self) -> str:
        return self.getAttribute('defaultValue')

    @defaultValue.setter
    def defaultValue(self, value: str):
        self.setAttribute('defaultValue', value)
        self.value = value

    @property
    def _radio_group(self) -> NodeList:
        if self.type.lower() == 'radio' and self.form and self.name:
            return NodeList([elm for elm in self.form
                             if elm.tagName == 'INPUT' and
                             elm.type.lower() == 'radio' and
                             elm.name == self.name])
        else:
            return NodeList([])


class HTMLLabelElement(HTMLElement, FormControlMixin):
    @property
    def htmlFor(self) -> str:
        return self.getAttribute('for')

    @htmlFor.setter
    def htmlFor(self, value: str):
        self.setAttribute('for', value)

    @property
    def control(self) -> HTMLElement:
        id = self.getAttribute('for')
        if id:
            if self.ownerDocument:
                return self.ownerDocument.getElementById(id)
            else:
                from wdom.document import getElementById
                return getElementById(id)


class HTMLOptGroupElement(HTMLElement, FormControlMixin):
    _special_attr_string = ['label']
    _special_attr_boolean = ['disabled']


class HTMLOptionElement(HTMLElement, FormControlMixin):
    _special_attr_string = ['label', 'value']
    _special_attr_boolean = ['defaultSelected', 'disabled', 'selected']


class HTMLScriptElement(HTMLElement):
    _special_attr_string = ['charset', 'src']
    _special_attr_boolean = ['async', 'defer']
    _should_escape_text = False


class HTMLSelectElement(HTMLElement, FormControlMixin):
    _special_attr_string = ['name', 'size', 'value']
    _special_attr_boolean = ['disabled', 'multiple', 'required']

    def __init__(self, *args, **kwargs):
        self._selected_options = []
        super().__init__(*args, **kwargs)

    def on_event_pre(self, e: Event):
        super().on_event_pre(e)
        if e.type in ('input', 'change'):
            self._set_attribute('value', e.currentTarget.get('value'))
            _selected = e.currentTarget.get('selectedOptions', [])
            self._selected_options.clear()
            for opt in self.options:
                if opt.rimo_id in _selected:
                    self._selected_options.append(opt)
                    opt._set_attribute('selected', True)
                else:
                    opt._remove_attribute('selected')

    @property
    def length(self) -> int:
        return len(self.options)

    @property
    def options(self) -> NodeList:
        return self.getElementsByTagName('option')

    @property
    def selectedOptions(self) -> NodeList:
        return NodeList(list(opt for opt in self.options if opt.selected))


class HTMLStyleElement(HTMLElement):
    _special_attr_boolean = ['disabled', 'scoped']
    _should_escape_text = False


class HTMLTextAreaElement(HTMLElement, FormControlMixin):
    _special_attr_string = ['height', 'name', 'src', 'value', 'width']
    _special_attr_boolean = ['disabled']
    defaultValue = HTMLElement.textContent

    def on_event_pre(self, e: Event):
        super().on_event_pre(e)
        if e.type in ('input', 'change'):
            # Update user inputs
            self._set_text_content(e.currentTarget.get('value') or '')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import unittest

from selenium.common.exceptions import NoSuchElementException

from wdom.tag import Tag, Textarea, Input, CheckBox, Div, Select, Option, Form
from wdom.tag import Label
from wdom.document import get_document
from wdom.misc import install_asyncio
from wdom.testing import RemoteBrowserTestCase, TestCase
from wdom.testing import start_remote_browser, close_remote_browser


def setUpModule():
    install_asyncio()
    start_remote_browser()


def tearDownModule():
    close_remote_browser()


class TestTag(RemoteBrowserTestCase, TestCase):
    if os.environ.get('TRAVIS', False):
        wait_time = 0.1

    def setUp(self):
        super().setUp()
        self.document = get_document()

        class Root(Tag):
            tag = 'root'
        self.root = Root()
        self.document.body.prepend(self.root)
        self.start()
        self.set_element(self.root)

    def test_connection(self):
        self.assertIsTrue(self.root.connected)
        # this is an example, but valid domain for test
        self.browser.get('http://example.com/')
        self.assertIsFalse(self.root.connected)

    def test_node_text(self):
        self.assertEqual(self.element.text, '')
        self.root.textContent = 'ROOT'
        self.wait_until(lambda: self.element.text == 'ROOT')
        self.assertEqual(self.element.text, 'ROOT')

    def test_node_attr(self):
        self.assertIsNone(self.element.get_attribute('src'))
        self.root.setAttribute('src', 'a')
        self.wait_until(lambda: self.element.get_attribute('src') == 'a')
        self.assertEqual(self.element.get_attribute('src'), 'a')
        self.root.removeAttribute('src')
        self.wait_until(lambda: self.element.get_attribute('src') is None)
        self.assertIsNone(self.element.get_attribute('src'))

    def test_node_class(self):
        self.root.addClass('a')
        self.wait_until(lambda: self.element.get_attribute('class') == 'a')
        self.assertEqual(self.element.get_attribute('class'), 'a')
        self.root.removeClass('a')
        self.wait_until(lambda: self.element.get_attribute('class') == '')
        self.assertEqual(self.element.get_attribute('class'), '')

    def test_addremove_child(self):
        child = Tag()
        self.root.appendChild(child)
        self.set_element(child)
        self.wait_until(lambda: self.element.text == '')
        self.assertEqual(self.element.text, '')
        child.textContent = 'Child'
        self.wait_until(lambda: self.element.text == 'Child')
        self.assertEqual(self.element.text, 'Child')

        self.root.removeChild(child)
        with self.assertRaises(NoSuchElementException):
            self.wait(0.1)
            self.set_element(child, self.wait_time * 10)

    def test_replace_child(self):
        child1 = Tag()
        child1.textContent = 'child1'
        child2 = Tag()
        child2.textContent = 'child2'
        self.root.appendChild(child1)
        with self.assertRaises(NoSuchElementException):
            self.set_element(child2, self.wait_time * 10)
        self.set_element(child1)
        self.wait_until(lambda: self.element.text == 'child1')
        self.assertEqual(self.element.text, 'child1')

        self.root.replaceChild(child2, child1)
        with self.assertRaises(NoSuchElementException):
            self.wait(0.1)
            self.set_element(child1, self.wait_time * 10)
        self.set_element(child2)
        self.wait_until(lambda: self.element.text == 'child2')
        self.assertEqual(self.element.text, 'child2')

    def test_showhide(self):
        self.root.textContent = 'root'
        self.wait_until(lambda: self.element.is_displayed())
        self.assertIsTrue(self.element.is_displayed())
        self.root.hide()
        self.wait_until(lambda: not self.element.is_displayed())
        self.assertIsFalse(self.element.is_displayed())
        self.root.show()
        self.wait_until(lambda: self.element.is_displayed())
        self.assertIsTrue(self.element.is_displayed())


class TestInput(RemoteBrowserTestCase, TestCase):
    if os.environ.get('TRAVIS', False):
        wait_time = 0.1

    def setUp(self):
        super().setUp()
        self.document = get_document(autoreload=False)
        self.root = Form()
        self.input = Input(parent=self.root, type='text')
        self.textarea = Textarea(parent=self.root)
        self.checkbox = CheckBox(parent=self.root, id='check1')
        self.check_l = Label('Check 1', parent=self.root, **{'for': 'check1'})
        self.radio1 = Input(parent=self.root, type='radio', name='radio_test', id='r1')  # noqa
        self.radio2 = Input(parent=self.root, type='radio', name='radio_test', id='r2')  # noqa
        self.radio3 = Input(parent=self.root, type='radio', name='radio_test2', id='r3')  # noqa
        self.radio1_l = Label('Radio 1', parent=self.root, **{'for': 'r1'})
        self.radio2_l = Label('Radio 2', parent=self.root, **{'for': 'r2'})
        self.radio3_l = Label('Radio 3', parent=self.root, **{'for': 'r3'})
        self.document.body.prepend(self.root)
        self.start()

    @unittest.skipIf(os.environ.get('TRAVIS', False),
                     reason='This test not pass only on travis')
    def test_textinput(self):
        self.set_element(self.input)
        self.element.send_keys('abc')
        self.wait_until(lambda: self.input.value == 'abc')
        self.assertEqual(self.input.value, 'abc')

        self.browser.get(self.url)
        self.set_element(self.input)
        self.wait_until(lambda: self.element.get_attribute('value') == 'abc')
        self.assertEqual(self.element.get_attribute('value'), 'abc')

        self.element.send_keys('def')
        self.wait_until(
            lambda: self.element.get_attribute('value') == 'abcdef')
        self.assertEqual(self.input.value, 'abcdef')

    @unittest.skipIf(os.environ.get('TRAVIS', False),
                     reason='This test not pass only on travis')
    def test_textarea(self):
        self.set_element(self.textarea)
        self.element.send_keys('abc')
        self.wait_until(lambda: self.textarea.value == 'abc')
        self.assertEqual(self.textarea.value, 'abc')

        self.browser.get(self.url)
        self.set_element(self.textarea)
        self.wait_until(lambda: self.element.get_attribute('value') == 'abc')
        self.assertEqual(self.element.get_attribute('value'), 'abc')

        self.element.send_keys('def')
        self.wait_until(
            lambda: self.element.get_attribute('value') == 'abcdef')
        self.assertEqual(self.textarea.value, 'abcdef')

    @unittest.skipIf(os.environ.get('TRAVIS', False),
                     reason='This test not pass only on travis')
    def test_checkbox(self):
        self.set_element(self.checkbox)
        self.element.click()
        self.wait_until(lambda: self.checkbox.checked)
        self.assertIsTrue(self.checkbox.checked)

        self.browser.get(self.url)
        self.set_element(self.checkbox)
        self.wait_until(
            lambda: self.element.get_attribute('checked') == 'true')
        self.assertEqual(self.element.get_attribute('checked'), 'true')

        self.element.click()
        self.wait_until(
            lambda: self.element.get_attribute('checked') is None)
        self.assertIsNone(self.element.get_attribute('checked'))
        self.assertIsFalse(self.checkbox.checked)

    @unittest.skipIf(os.environ.get('TRAVIS', False),
                     reason='This test not pass only on travis')
    def test_checkbox_label(self):
        self.set_element(self.check_l)
        self.element.click()
        self.wait_until(lambda: self.checkbox.checked)
        self.assertTrue(self.checkbox.checked)

        self.element.click()
        self.wait_until(lambda: not self.checkbox.checked)
        self.assertFalse(self.checkbox.checked)

    @unittest.skipIf(os.environ.get('TRAVIS', False),
                     reason='This test not pass only on travis')
    def test_radios(self):
        self.assertFalse(self.radio1.checked)
        self.assertFalse(self.radio2.checked)
        self.assertFalse(self.radio3.checked)

        self.set_element(self.radio1)
        self.element.click()
        self.wait_until(lambda: self.radio1.checked)
        self.assertTrue(self.radio1.checked)
        self.assertFalse(self.radio2.checked)
        self.assertFalse(self.radio3.checked)

        self.set_element(self.radio2)
        self.element.click()
        self.wait_until(lambda: self.radio2.checked)
        self.assertFalse(self.radio1.checked)
        self.assertTrue(self.radio2.checked)
        self.assertFalse(self.radio3.checked)

        self.set_element(self.radio3)
        self.element.click()
        self.wait_until(lambda: self.radio3.checked)
        self.assertFalse(self.radio1.checked)
        self.assertTrue(self.radio2.checked)
        self.assertTrue(self.radio3.checked)

    @unittest.skipIf(os.environ.get('TRAVIS', False),
                     reason='This test not pass only on travis')
    def test_radios_label(self):
        self.set_element(self.radio1_l)
        self.element.click()
        self.wait_until(lambda: self.radio1.checked)
        self.assertTrue(self.radio1.checked)
        self.assertFalse(self.radio2.checked)

        self.set_element(self.radio2_l)
        self.element.click()
        self.wait_until(lambda: self.radio2.checked)
        self.assertFalse(self.radio1.checked)
        self.assertTrue(self.radio2.checked)


class TestSelect(RemoteBrowserTestCase, TestCase):
    if os.environ.get('TRAVIS', False):
        wait_time = 0.1

    def setUp(self):
        super().setUp()
        self.document = get_document(autoreload=False)
        self.root = Div()
        self.select = Select(parent=self.root)
        self.mselect = Select(parent=self.root, multiple=True)
        self.opt1 = Option('option 1', parent=self.select)
        self.opt2 = Option('option 2', parent=self.select)
        self.opt3 = Option('option 3', parent=self.select, value='opt3')
        self.opt1m = Option('option 1', parent=self.mselect)
        self.opt2m = Option('option 2', parent=self.mselect)
        self.opt3m = Option('option 3', parent=self.mselect, value='opt3m')
        self.document.body.prepend(self.root)
        self.start()

    @unittest.skipIf(os.environ.get('TRAVIS', False),
                     reason='This test not pass only on travis')
    def test_select(self):
        self.set_element(self.select)
        self.element.select_by_index(1)
        self.wait_until(lambda: self.opt2.selected)
        self.assertEqual(self.select.value, 'option 2')
        self.assertFalse(self.opt1.selected)
        self.assertTrue(self.opt2.selected)
        self.assertFalse(self.opt3.selected)

        self.element.select_by_visible_text('option 1')
        self.wait_until(lambda: self.opt1.selected)
        self.assertEqual(self.select.value, 'option 1')
        self.assertTrue(self.opt1.selected)
        self.assertFalse(self.opt2.selected)
        self.assertFalse(self.opt3.selected)

        self.element.select_by_value('opt3')
        self.wait_until(lambda: self.opt3.selected)
        self.assertEqual(self.select.value, 'opt3')
        self.assertFalse(self.opt1.selected)
        self.assertFalse(self.opt2.selected)
        self.assertTrue(self.opt3.selected)

    def test_multi_select(self):
        self.set_element(self.mselect)
        self.element.select_by_index(1)
        self.wait_until(lambda: self.opt2m.selected)
        self.assertEqual(self.mselect.value, 'option 2')
        self.assertFalse(self.opt1m.selected)
        self.assertTrue(self.opt2m.selected)
        self.assertFalse(self.opt3m.selected)

        self.element.select_by_index(2)
        self.wait_until(lambda: self.opt3m.selected)
        self.assertFalse(self.opt1m.selected)
        self.assertTrue(self.opt2m.selected)
        self.assertTrue(self.opt3m.selected)

        self.element.deselect_all()
        self.wait_until(lambda: not self.opt3m.selected)
        self.assertFalse(self.opt1m.selected)
        self.assertFalse(self.opt2m.selected)
        self.assertFalse(self.opt3m.selected)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from wdom.tag import NewTagClass as NewTag
from wdom.tag import *

name = 'Milligram'
project_url = 'http://milligram.github.io/'
project_repository = 'https://github.com/milligram/milligram'
license = 'MIT License'
license_url = 'https://github.com/milligram/milligram/blob/master/license'

css_files = [
    '//fonts.googleapis.com/css?family=Roboto:300,300italic,700,700italic',
    '//cdnjs.cloudflare.com/ajax/libs/normalize/3.0.3/normalize.css',
    '//cdnjs.cloudflare.com/ajax/libs/milligram/1.1.0/milligram.min.css'
]

DefaultButton = NewTag('DefaultButton', 'button', Button, class_='button-outline')
InfoButton = NewTag('InfoButton', 'button', Button, class_='button-outline')
PrimaryButton = NewTag('PrimaryButton', 'button', Button, class_='button-primary')
LinkButton = NewTag('LinkButton', 'button', Button, class_='button-clear')
Button = NewTag('Button', 'button', DefaultButton, class_='button-outline')

Container = NewTag('Container', 'div', Container, class_='container')
Wrapper = NewTag('Wrapper', 'div', Wrapper, class_='container')
Row = NewTag('Row', 'div', Row, class_='row')
Col = NewTag('Col', 'div', Col, class_='column')
Col1 = NewTag('Col1', 'div', Col1, class_='column-10 column')
Col2 = NewTag('Col2', 'div', Col2, class_='column-20 column')
Col3 = NewTag('Col3', 'div', Col3, class_='column-25 column')
Col4 = NewTag('Col4', 'div', Col4, class_='column-33 column')
Col5 = NewTag('Col5', 'div', Col5, class_='column-41 column')
Col6 = NewTag('Col6', 'div', Col6, class_='column-50 column')
Col7 = NewTag('Col7', 'div', Col7, class_='column-60 column')
Col8 = NewTag('Col8', 'div', Col8, class_='column-67 column')
Col9 = NewTag('Col9', 'div', Col9, class_='column-75 column')
Col10 = NewTag('Col10', 'div', Col10, class_='column-80 column')
Col11 = NewTag('Col11', 'div', Col11, class_='column-90 column')
Col12 = NewTag('Col12', 'div', Col12, class_='column-100 column')

extended_classes = [
    DefaultButton,
    InfoButton,
    PrimaryButton,
    LinkButton,
    Button,
    Container,
    Wrapper,
    Row,
    Col,
    Col1,
    Col2,
    Col3,
    Col4,
    Col5,
    Col6,
    Col7,
    Col8,
    Col9,
    Col10,
    Col11,
    Col12,
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import asyncio
from unittest.mock import MagicMock, call

from syncer import sync

from wdom.interface import Event
from wdom.testing import TestCase
from wdom.web_node import WebElement
from wdom.server import set_server_type, _tornado


class TestWebElement(TestCase):
    def setUp(self):
        self.elm = WebElement('tag')
        self.c1 = WebElement()
        self.c2 = WebElement()
        self.js_mock = MagicMock()
        self.js_mock1 = MagicMock()
        self.js_mock2 = MagicMock()
        self.elm.js_exec = self.js_mock
        self.c1.js_exec = self.js_mock1
        self.c2.js_exec = self.js_mock2

    def test_id(self):
        self.assertRegex(self.elm.html, r'<tag rimo_id="\d+"></tag>')
        self.assertRegex(self.elm.rimo_id, r'\d+')

    def test_noid(self):
        self.assertEqual('<tag></tag>', self.elm.html_noid)

        self.c1.tag = 'c1'
        appended_child1 = self.elm.appendChild(self.c1)
        self.assertIs(appended_child1, self.c1)
        self.assertEqual('<tag><c1></c1></tag>', self.elm.html_noid)

        self.c2.tag = 'c2'
        appended_child2 = self.c1.appendChild(self.c2)
        self.assertIs(appended_child2, self.c2)
        self.assertEqual('<tag><c1><c2></c2></c1></tag>', self.elm.html_noid)

    def test_id_init(self):
        elm = WebElement('tag', rimo_id='myid')
        self.assertEqual('<tag rimo_id="myid"></tag>', elm.html)

    def test_not_connected(self):
        self.assertFalse(self.elm.connected)

    def test_parent(self):
        self.assertIsNone(self.elm.parentNode)
        self.assertIsNone(self.c1.parentNode)
        self.elm.appendChild(self.c1)
        self.assertIs(self.elm, self.c1.parentNode)
        self.js_mock1.assert_not_called()
        self.assertEqual(self.js_mock.call_count, 1)
        self.js_mock.assert_called_once_with(
            'insertAdjacentHTML', 'beforeend', self.c1.html)

        removed_child1 = self.elm.removeChild(self.c1)
        self.assertIs(removed_child1, self.c1)
        self.assertIsNone(self.c1.parentNode)
        self.assertEqual(self.js_mock.call_count, 2)
        self.js_mock1.assert_not_called()
        self.js_mock.assert_called_with('removeChildById', self.c1.rimo_id)

    def test_addremove_child(self):
        self.assertFalse(self.elm.hasChildNodes())
        self.elm.appendChild(self.c1)
        self.assertTrue(self.elm.hasChildNodes())
        self.assertIn(self.c1, self.elm)
        self.assertNotIn(self.c2, self.elm)
        self.assertEqual(self.elm.length, 1)

        self.elm.insertBefore(self.c2, self.c1)
        self.assertIn(self.c1, self.elm)
        self.assertIn(self.c2, self.elm)
        self.assertEqual(self.elm.length, 2)
        self.c2.remove()
        self.assertEqual(self.elm.length, 1)
        self.assertIn(self.c1, self.elm)
        self.assertNotIn(self.c2, self.elm)
        self.assertIsNone(self.c2.parentNode)
        self.js_mock2.assert_called_once_with('remove')

        self.elm.removeChild(self.c1)
        self.assertFalse(self.elm.hasChildNodes())
        self.assertEqual(self.elm.length, 0)
        self.assertNotIn(self.c1, self.elm)
        self.assertNotIn(self.c2, self.elm)
        self.js_mock1.assert_called_once_with(
            'insertAdjacentHTML', 'beforebegin', self.c2.html)
        self.assertEqual(self.js_mock.call_count, 3)

        with self.assertRaises(ValueError):
            self.elm.removeChild(self.c1)

    def test_addremove_attr(self):
        self.elm.setAttribute('src', 'a')
        self.js_mock.assert_called_with('setAttribute', 'src', 'a')
        self.elm.removeAttribute('src')
        self.js_mock.assert_called_with('removeAttribute', 'src')

    def test_style(self):
        self.elm.style = 'color: red;'
        self.js_mock.assert_called_once_with(
            'setAttribute', 'style', 'color: red;')
        self.elm.removeAttribute('style')
        self.js_mock.assert_called_with('removeAttribute', 'style')
        self.elm.style.color = 'black'
        self.js_mock.assert_called_with(
            'setAttribute', 'style', 'color: black;')

    def test_style_init(self):
        _js_exec = WebElement.js_exec
        WebElement.js_exec = self.js_mock
        WebElement('elm', style='color: red;')
        _call = call('setAttribute', 'style', 'color: red;')
        _call_remove = call('removeAttribute', 'style')
        self.js_mock.assert_has_calls([_call])
        count = 0
        for c in self.js_mock.call_args_list:
            if c == _call:
                count += 1
            elif c == _call_remove:
                raise AssertionError('Unexpeted remove style')
        self.assertEqual(count, 1)
        WebElement.js_exec = _js_exec

    def test_set_text_content(self):
        self.elm.textContent = 'text'
        self.js_mock.assert_called_once_with('textContent', 'text')

    def test_set_inner_html(self):
        self.elm.innerHTML = 'html'
        self.js_mock.assert_called_once_with('innerHTML', 'html')

    def test_shallow_copy(self):
        from copy import copy
        clone = copy(self.elm)
        self.assertNotEqual(clone.rimo_id, self.elm.rimo_id)

        clone = self.elm.cloneNode()
        self.assertNotEqual(clone.rimo_id, self.elm.rimo_id)

    def test_deep_copy(self):
        from copy import deepcopy
        clone = deepcopy(self.elm)
        self.assertNotEqual(clone.rimo_id, self.elm.rimo_id)

        clone = self.elm.cloneNode(deep=True)
        self.assertNotEqual(clone.rimo_id, self.elm.rimo_id)

    def test_click(self):
        mock = MagicMock(_is_coroutine=False)
        self.elm.addEventListener('click', mock)
        self.elm.click()
        self.js_mock.assert_called_once_with('addEventListener', 'click')
        self.assertEqual(mock.call_count, 1)


class TestEventMessage(TestCase):
    def setUp(self):
        self.elm = WebElement('tag')
        self.elm.js_exec = MagicMock()
        self.mock = MagicMock(_is_coroutine=False)
        self.elm.addEventListener('click', self.mock)
        self.msg = {
            'type': 'event',
            'id': self.elm.rimo_id,
            'event': {
                'type': 'click',
            },
        }
        self.event = Event(**self.msg.get('event'))

    def test_handle_event(self):
        self.elm.js_exec.assert_called_once_with('addEventListener', 'click')
        self.elm.dispatchEvent(self.event)
        self.assertTrue(self.mock.called)

    def test_remove_event(self):
        self.elm.removeEventListener('click', self.mock)
        self.elm.js_exec.assert_called_with('removeEventListener', 'click')
        self.elm.dispatchEvent(self.event)
        self.mock.assert_not_called()


class TestQuery(TestCase):
    def setUp(self):
        super().setUp()
        set_server_type('tornado')
        _tornado.connections.append(MagicMock())
        self.elm = WebElement('tag')
        self.elm.js_exec = MagicMock()
        self.msg = {
            'type': 'response',
            'id': self.elm.rimo_id,
        }

    def test_query(self):
        fut = self.elm.js_query('test')
        self.elm.js_exec.assert_called_once_with('test', 0)
        self.msg['reqid'] = 0
        self.msg['data'] = 1
        self.elm.on_response(self.msg)
        self.assertEqual(fut.result(), 1)

    @sync
    @asyncio.coroutine
    def test_scroll(self):
        fut = self.elm.scrollX()
        self.assertFalse(fut.done())
        self.msg['reqid'] = 0
        self.msg['data'] = {'x': 1}
        self.elm.on_response(self.msg)
        x = yield from fut
        self.assertEqual(x, {'x': 1})



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from collections import defaultdict
from asyncio import ensure_future, iscoroutinefunction
from typing import Callable

from wdom.interface import Event
from wdom.webif import WebIF


class EventListener:
    '''Class to wrap an event listener. Acceptable listeners are function,
    coroutine, and coroutine-function. If ``apply_data`` is True, ``data`` is
    applied as a keyword argument of ``data`` when the registered event is
    triggered.'''
    # Should support generator?
    def __init__(self, listener: Callable[[Event], None]):
        self.listener = listener

        if iscoroutinefunction(self.listener):
            self.action = self.wrap_coro_func(self.listener)
            self._is_coroutine = True
        else:
            self.action = self.listener
            self._is_coroutine = False

    def wrap_coro_func(self, coro: Callable[[Event], None]
                       ) -> Callable[[Event], None]:
        def wrapper(*args, **kwargs):
            nonlocal coro
            return ensure_future(coro(*args, **kwargs))
        return wrapper

    def __call__(self, event: Event):
        return self.action(event)


class EventTarget:
    def __init__(self, *args, **kwargs):
        self._listeners = defaultdict(list)
        super().__init__(*args, **kwargs)

    def _add_event_listener(self, event: str,
                            listener: Callable[[Event], None]):
        self._listeners[event].append(EventListener(listener))

    def _add_event_listener_web(self, event: str, *args, **kwargs):
        if isinstance(self, WebIF):
            self.js_exec('addEventListener', event)

    def addEventListener(self, event: str,
                         listener: Callable[[Event], None]):
        '''Add event listener to this node. ``event`` is a string which
        determines the event type when the new listener called. Acceptable
        events are same as JavaScript, without ``on``. For example, to add a
        listener which is called when this node is clicked, event is
        ``'click``.
        '''
        self._add_event_listener(event, listener)
        self._add_event_listener_web(event)

    def _remove_event_listener(self, event: str,
                               listener: Callable[[Event], None]):
        listeners = self._listeners[event]
        if not listeners:
            return
        for l in listeners:
            if l.listener == listener:
                listeners.remove(l)
                break
        if not listeners:
            del self._listeners[event]

    def _remove_event_listener_web(self, event: str, *args, **kwargs):
        if isinstance(self, WebIF) and event not in self._listeners:
            self.js_exec('removeEventListener', event)

    def removeEventListener(self, event: str,
                            listener: Callable[[Event], None]):
        '''Remove an event listener of this node. The listener is removed only
        when both event type and listener is matched.
        '''
        self._remove_event_listener(event, listener)
        self._remove_event_listener_web(event)

    def _dispatch_event(self, event: Event):
        _tasks = []
        for listener in self._listeners[event.type]:
            if listener._is_coroutine:
                _tasks.append(listener(event))
            else:
                listener(event)
        return _tasks

    def dispatchEvent(self, event: Event):
        return self._dispatch_event(event)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from wdom.tag import Tag, H1, Div
from wdom.document import get_document


def sample_page() -> Tag:
    doc = get_document()
    text = H1(parent=doc.body)
    text.textContent = 'Title'
    box = Div(style='width:200px;height:10000px;background-color:red;')
    doc.body.appendChild(box)

    def scroll(data):
        box.scrollBy(100, 100)

    box.addEventListener('click', scroll)

    return doc



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from wdom.tag import NewTagClass as NewTag
from wdom.tag import *

name = 'Spark'
project_url = 'http://codewithspark.com/'
project_repository = 'https://github.com/twistedpixel/spark/'
license = 'MIT License'
license_url = 'https://github.com/twistedpixel/spark/blob/master/dist/LICENSE'

css_files = [
    '_static/css/spark.min.css',
]

js_files = [
    '//ajax.googleapis.com/ajax/libs/jquery/1.11.2/jquery.min.js',
    '_static/js/spark.min.js',
]

headers = []

Button = NewTag('Button', bases=Button, class_='btn flat')
DefaultButton = NewTag('DefaultButton', 'button', Button, class_='black', is_='default-button')
PrimaryButton = NewTag('PrimaryButton', 'button', Button, class_='blue', is_='primary-button')
SecondaryButton = NewTag('SecondaryButton', 'button', Button, class_='green', is_='secondary-button')
SuccessButton = NewTag('SuccessButton', 'button', Button, class_='lime', is_='success-button')
InfoButton = NewTag('InfoButton', 'button', Button, class_='green', is_='info-button')
WarningButton = NewTag('WarningButton', 'button', Button, class_='orange', is_='warning-button')
DangerButton = NewTag('DangerButton', 'button', Button, class_='red', is_='danger-button')
ErrorButton = NewTag('ErrorButton', 'button', Button, class_='red', is_='error-button')
LinkButton = NewTag('LinkButton', 'button', Button, class_='flat', is_='link-button')

class Form(Form):
    class_ = 'form'
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setAttribute('role', 'form')

Table = NewTag('Table', 'table', Table, class_='tbl')

Container = NewTag('Container', 'div', Container, class_='container')
Wrapper = NewTag('Wrapper', 'div', Wrapper, class_='container')
Row = NewTag('Row', 'div', Row, class_='row')
Col = NewTag('Col', 'div', Col, class_='col')
Col1 = NewTag('Col1', 'div', Col1, class_='col of-1')
Col2 = NewTag('Col2', 'div', Col2, class_='col of-2')
Col3 = NewTag('Col3', 'div', Col3, class_='col of-3')
Col4 = NewTag('Col4', 'div', Col4, class_='col of-4')
Col5 = NewTag('Col5', 'div', Col5, class_='col of-5')
Col6 = NewTag('Col6', 'div', Col6, class_='col of-6')
Col7 = NewTag('Col7', 'div', Col7, class_='col of-7')
Col8 = NewTag('Col8', 'div', Col8, class_='col of-8')
Col9 = NewTag('Col9', 'div', Col9, class_='col of-9')
Col10 = NewTag('Col10', 'div', Col10, class_='col of-10')
Col11 = NewTag('Col11', 'div', Col11, class_='col of-11')
Col12 = NewTag('Col12', 'div', Col12, class_='col of-12')

extended_classes = [
    Button,
    DefaultButton,
    PrimaryButton,
    SecondaryButton,
    SuccessButton,
    InfoButton,
    WarningButton,
    DangerButton,
    ErrorButton,
    LinkButton,
    Table,
    Container,
    Wrapper,
    Row,
    Col,
    Col1,
    Col2,
    Col3,
    Col4,
    Col5,
    Col6,
    Col7,
    Col8,
    Col9,
    Col10,
    Col11,
    Col12,
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from functools import partial

from wdom.tag import Tag, H1, Button
from wdom.document import get_document


def sample_page() -> Tag:
    doc = get_document()
    text = H1(parent=doc.body)
    text.textContent = 'Title!'
    btn = Button('click!', parent=doc.body)

    def get_rect(data):
        print('send query')
        def show_rect(fut):
            print('DONE')
            print(fut.result())
        fut = text.getBoundingClientRect()
        fut.add_done_callback(show_rect)
        print(fut)

    btn.addEventListener('click', get_rect)

    return doc



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from wdom.tag import NewTagClass as NewTag
from wdom.tag import *

name = 'Picnic'
project_url = 'http://picnicss.com/'
project_repository = 'https://github.com/picnicss/picnic'
license = 'MIT License'
license_url = 'https://github.com/picnicss/picnic/blob/master/LICENSE'

css_files = [
    '//cdn.jsdelivr.net/picnicss/5.1.0/picnic.min.css',
]

Button = NewTag('Button', bases=Button)
DefaultButton = NewTag('DefaultButton', 'button', Button, is_='default-button')
PrimaryButton = NewTag('PrimaryButton', 'button', Button, is_='primary-button')
SecondaryButton = NewTag('SecondaryButton', 'button', Button, is_='secondary-button')
SuccessButton = NewTag('SuccessButton', 'button', Button, class_='success', is_='success-button')
InfoButton = NewTag('InfoButton', 'button', Button, is_='info-button')
WarningButton = NewTag('WarningButton', 'button', Button, class_='warning', is_='warning-button')
DangerButton = NewTag('DangerButton', 'button', Button, class_='error', is_='danger-button')
ErrorButton = NewTag('ErrorButton', 'button', Button, class_='error', is_='error-button')
LinkButton = NewTag('LinkButton', 'button', Button, class_='pseudo', is_='link-button')

Row = NewTag('Row', 'div', Row, class_='row')
Col1 = NewTag('Col1', 'div', Col1)
Col2 = NewTag('Col2', 'div', Col2)
Col3 = NewTag('Col3', 'div', Col3, class_='fourth')
Col4 = NewTag('Col4', 'div', Col4, class_='third')
Col5 = NewTag('Col5', 'div', Col5)
Col6 = NewTag('Col6', 'div', Col6, class_='half')
Col7 = NewTag('Col7', 'div', Col7)
Col8 = NewTag('Col8', 'div', Col8, class_='two-third')
Col9 = NewTag('Col9', 'div', Col9, class_='three-fourth')
Col10 = NewTag('Col10', 'div', Col10)
Col11 = NewTag('Col11', 'div', Col11)
Col12 = NewTag('Col12', 'div', Col12)

extended_classes = [
    Button,
    DefaultButton,
    PrimaryButton,
    SecondaryButton,
    SuccessButton,
    InfoButton,
    WarningButton,
    DangerButton,
    ErrorButton,
    LinkButton,
    Row,
    Col1,
    Col2,
    Col3,
    Col4,
    Col5,
    Col6,
    Col7,
    Col8,
    Col9,
    Col10,
    Col11,
    Col12,
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import re
from wdom.options import config
from wdom.themes import theme_list, logger

theme = config.theme

if theme:
    if theme.endswith('.py'):
        theme = re.sub(r'\.py[codx]?$', '', theme)
    if theme in theme_list:
        # import theme
        logger.info('Use theme: {}'.format(theme))
        exec('from wdom.themes.{} import *'.format(theme))
    else:
        logger.warning(
            'Unknown theme "{}" was specified. Use default.'.format(theme))
        # Remove duplicated module name (*.py and *.pyc may exists)
        logger.warning(
            'Available themes: {}'.format(', '.join(theme_list)))
        from wdom.tag import *
else:
    # Use default
    logger.info('No theme specified. Use default theme.')
    from wdom.tag import *
    name = 'default'
    project_url = ''
    project_repository = ''
    license = ''
    license_url = ''
    css_files = []
    js_files = []
    headers = []



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from wdom.tag import NewTagClass as NewTag
from wdom.tag import *

name = 'Semantic UI (Semantic)'
project_url = 'http://semantic-ui.com/'
project_repository = 'https://github.com/semantic-org/semantic-ui/'
license = 'MIT License'
license_url = 'https://github.com/Semantic-Org/Semantic-UI/blob/master/LICENSE.md'

css_files = [
    '//cdnjs.cloudflare.com/ajax/libs/semantic-ui/2.1.8/semantic.min.css',
]

# Need jQuery
js_files = [
    '//ajax.googleapis.com/ajax/libs/jquery/1.11.2/jquery.min.js',
    '//cdnjs.cloudflare.com/ajax/libs/semantic-ui/2.1.8/semantic.min.js',
]

headers = []

# Buttons http://semantic-ui.com/elements/button.html
Button = NewTag('Button', bases=Button, class_='ui button')
DefaultButton = NewTag('DefaultButton', 'button', Button, is_='default-button')
PrimaryButton = NewTag('PrimaryButton', 'button', Button, class_='primary', is_='primary-button')
SecondaryButton = NewTag('SecondaryButton', 'button', Button, class_='blue', is_='secondary-button')
SuccessButton = NewTag('SuccessButton', 'button', Button, class_='teal', is_='success-button')
InfoButton = NewTag('InfoButton', 'button', Button, class_='green', is_='info-button')
WarningButton = NewTag('WarningButton', 'button', Button, class_='orange', is_='warning-button')
DangerButton = NewTag('DangerButton', 'button', Button, class_='red', is_='danger-button')
ErrorButton = NewTag('ErrorButton', 'button', Button, class_='red', is_='error-button')
LinkButton = NewTag('LinkButton', 'button', Button, class_='basic', is_='link-button')

# Form http://semantic-ui.com/collections/form.html
Form = NewTag('Form', 'form', Form, class_='ui form')
# DropDown http://semantic-ui.com/modules/dropdown.html
# document is not correct...
# Select = NewTag('Select', 'select', Select, class_='ui fluid dropdown')
Select = NewTag('Select', 'select', Select, class_='ui dropdown')

# List http://semantic-ui.com/elements/list.html
# Nested list with ul.list fails...
Ul = NewTag('Ul', 'div', Div, class_='ui bulleted list')
Li = NewTag('Li', 'div', Div, class_='item')
Ol = NewTag('Ol', 'div', Div, class_='ui ordered list')

# Table http://semantic-ui.com/collections/table.html
Table = NewTag('Table', 'table', Table, class_='ui celled table')

# Divider
Hr = NewTag('Hr', 'div', Div, class_='ui divider')

# Typography http://semantic-ui.com/elements/header.html
H1 = NewTag('H1', 'h1', H1, class_='ui header')
H2 = NewTag('H2', 'h2', H2, class_='ui header')
H3 = NewTag('H3', 'h3', H3, class_='ui header')
H4 = NewTag('H4', 'h4', H4, class_='ui header')
H5 = NewTag('H5', 'h5', H5, class_='ui header')
H6 = NewTag('H6', 'h6', H6, class_='ui header')

# Grid http://semantic-ui.com/collections/grid.html
# Too complicatted...
Container = NewTag('Container', 'div', Container, class_='ui grid container')
Wrapper = NewTag('Wrapper', 'div', Wrapper, class_='ui grid container')
Row = NewTag('Row', 'div', Row, class_='row')
Col = NewTag('Col', 'div', Col, class_='column')
Col1 = NewTag('Col1', 'div', Col1, class_='column')
Col2 = NewTag('Col2', 'div', Col2, class_='two wide column')
Col3 = NewTag('Col3', 'div', Col3, class_='three wide column')
Col4 = NewTag('Col4', 'div', Col4, class_='four wide column')
Col5 = NewTag('Col5', 'div', Col5, class_='five wide column')
Col6 = NewTag('Col6', 'div', Col6, class_='six wide column')
Col7 = NewTag('Col7', 'div', Col7, class_='seven wide column')
Col8 = NewTag('Col8', 'div', Col8, class_='eight wide column')
Col9 = NewTag('Col9', 'div', Col9, class_='nine wide column')
Col10 = NewTag('Col10', 'div', Col10, class_='ten wide column')
Col11 = NewTag('Col11', 'div', Col11, class_='eleven wide column')
Col12 = NewTag('Col12', 'div', Col12, class_='')

extended_classes = [
    Button,
    DefaultButton,
    PrimaryButton,
    SecondaryButton,
    SuccessButton,
    InfoButton,
    WarningButton,
    DangerButton,
    ErrorButton,
    LinkButton,
    Form,
    Select,
    Ul,
    Li,
    Ol,
    Table,
    Hr,
    H1,
    H2,
    H3,
    H4,
    H5,
    H6,
    Container,
    Wrapper,
    Row,
    Col,
    Col1,
    Col2,
    Col3,
    Col4,
    Col5,
    Col6,
    Col7,
    Col8,
    Col9,
    Col10,
    Col11,
    Col12,
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from os import path
import unittest
from wdom.testing import suppress_logging


def main():
    suppress_logging()
    target_dir = path.dirname(__file__)
    tests = unittest.defaultTestLoader.discover(target_dir)
    runner = unittest.TextTestRunner()
    runner.run(tests)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from wdom.tag import NewTagClass as NewTag
from wdom.tag import *

name = 'INK'
project_url = 'http://ink.sapo.pt/'
project_repository = 'https://github.com/sapo/Ink/'
license = 'MIT License'
license_url = 'https://github.com/sapo/Ink/blob/develop/LICENSE'

css_files = [
    '//fastly.ink.sapo.pt/3.1.10/css/ink.css',
]
js_files = [
    '//fastly.ink.sapo.pt/3.1.10/js/ink-all.js',
]
headers = []

Button = NewTag('Button', 'button', Button, class_='ink-button')
DefaultButton = NewTag('DefaultButton', 'button', Button, is_='default-button')
PrimaryButton = NewTag('PrimaryButton', 'button', Button, class_='blue', is_='primary-button')
SecondaryButton = NewTag('SecondaryButton', 'button', Button, is_='secondary-button')
SuccessButton = NewTag('SuccessButton', 'button', Button, class_='green', is_='success-button')
InfoButton = NewTag('InfoButton', 'button', Button, class_='black', is_='info-button')
WarningButton = NewTag('WarningButton', 'button', Button, class_='orange', is_='warning-button')
DangerButton = NewTag('DangerButton', 'button', Button, class_='red', is_='danger-button')
ErrorButton = NewTag('ErrorButton', 'button', Button, class_='red', is_='error-button')
LinkButton = NewTag('LinkButton', 'button', Button, is_='link-button')

Form = NewTag('Form', 'form', Form, class_='ink-form')
FormGroup = NewTag('FormGroup', 'div', Div, class_='control-group')
Input = NewTag('Input', 'input', Input)
TextInput = NewTag('TextInput', 'input', TextInput)
Textarea = NewTag('Textarea', 'textarea', Textarea)
Select = NewTag('Select', 'ul', Ul, class_='dropdown-menu')
Option = NewTag('Option', 'li', Li)

Table = NewTag('Table', 'table', Table, class_='ink-table')

Container = NewTag('Container', 'div', Container, class_='ink-grid')
Wrapper = NewTag('Wrapper', 'div', Wrapper, class_='ink-grid')
Row = NewTag('Row', 'div', Row, class_='column-group')
Col1 = NewTag('Col1', 'div', Col1, class_='all-10')
Col2 = NewTag('Col2', 'div', Col2, class_='all-15')
Col3 = NewTag('Col3', 'div', Col3, class_='all-25')
Col4 = NewTag('Col4', 'div', Col4, class_='all-33')
Col5 = NewTag('Col5', 'div', Col5, class_='all-40')
Col6 = NewTag('Col6', 'div', Col6, class_='all-50')
Col7 = NewTag('Col7', 'div', Col7, class_='all-60')
Col8 = NewTag('Col8', 'div', Col8, class_='all-66')
Col9 = NewTag('Col9', 'div', Col9, class_='all-75')
Col10 = NewTag('Col10', 'div', Col10, class_='all-85')
Col11 = NewTag('Col11', 'div', Col11, class_='all-90')
Col12 = NewTag('Col12', 'div', Col12, class_='all-100')

extended_classes = [
    Button,
    DefaultButton,
    PrimaryButton,
    SecondaryButton,
    SuccessButton,
    InfoButton,
    WarningButton,
    DangerButton,
    ErrorButton,
    LinkButton,
    Form,
    FormGroup,
    Input,
    TextInput,
    Textarea,
    Select,
    Option,
    Table,
    Container,
    Wrapper,
    Row,
    Col1,
    Col2,
    Col3,
    Col4,
    Col5,
    Col6,
    Col7,
    Col8,
    Col9,
    Col10,
    Col11,
    Col12,
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from wdom.interface import Node
from wdom.element import Element
from wdom.tag import Tag, default_classes


class CustomElementsRegistry(dict):
    '''Keep elements by (name (custom-tag or is-attr), extended-tag (or None))
    pair.
    '''
    def _upgrage_to_tag_class(self, elm):
        if elm.type_ and 'type' not in elm.attributes:
            elm.setAttribute('type', elm.type_)
        if elm.is_ and 'is' not in elm.attributes:
            elm.setAttribute('is', elm.is_)

    def _upgrade_by_tag(self, name: str, constructor: type):
        for elm in Element._elements:
            if not elm._registered and elm.tag == name:
                elm.__class__ = constructor
                elm._registered = True
                if isinstance(elm, Tag):
                    self._upgrage_to_tag_class(elm)

    def _upgrade_by_is(self, name: str, constructor: type, extends: str):
        for elm in Element._elements:
            if (not elm._registered and elm.tag == extends and
                    elm.getAttribute('is') == name):
                elm.__class__ = constructor
                elm._registered = True
                if isinstance(elm, Tag):
                    self._upgrage_to_tag_class(elm)

    def _define(self, name: str, constructor: type, options: dict = None):
        extends = options.get('extends').lower() if options else None
        self[(name, extends)] = constructor
        if extends:
            self._upgrade_by_is(name, constructor, extends)
        else:
            self._upgrade_by_tag(name, constructor)

    def _define_orig(self, name: str, constructor: type, options: dict = None):
        self._define(name.lower(), constructor, options)

    def _define_class(self, constructor: type):
        is_ = getattr(constructor, 'is_', getattr(constructor, 'is', None))
        if is_:
            name = is_.lower()
            options = {'extends': constructor.tag}
        else:
            name = constructor.tag.lower()
            options = {}
        self._define(name, constructor, options)

    def define(self, *args, **kwargs):
        if isinstance(args[0], str):
            self._define_orig(*args, **kwargs)
        elif isinstance(args[0], type):
            self._define_class(*args, **kwargs)
        else:
            raise TypeError(
                'Invalid argument for define: {}, {}'.format(args, kwargs))

    def _define_default(self):
        for cls in default_classes:
            self.define(cls)

    def reset(self):
        self.clear()
        self._define_default()


customElements = CustomElementsRegistry()
customElements._define_default()


class Window:
    @property
    def document(self) -> Node:
        return self._document

    @property
    def customElements(self) -> CustomElementsRegistry:
        return self._custom_elements

    def __init__(self, document: Node):
        self._document = document
        self._custom_elements = customElements



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from wdom.tag import NewTagClass as NewTag
from wdom.tag import *

name = 'Bulma'
project_url = 'http://bulma.io/'
project_repository = 'https://github.com/jgthms/bulma/'
license = 'MIT License'
license_url = 'https://github.com/jgthms/bulma/blob/master/LICENSE'

css_files = [
    '//cdnjs.cloudflare.com/ajax/libs/bulma/0.0.20/css/bulma.min.css',
]

js_files = []
headers = []

Button = NewTag('Button', bases=Button, class_='button')
DefaultButton = NewTag('DefaultButton', 'button', Button, is_='default-button')
PrimaryButton = NewTag('PrimaryButton', 'button', Button, class_='is-primary', is_='primary-button')
SecondaryButton = NewTag('SecondaryButton', 'button', PrimaryButton, class_='is-outlined', is_='secondary-button')
SuccessButton = NewTag('SuccessButton', 'button', Button, class_='is-success', is_='success-button')
InfoButton = NewTag('InfoButton', 'button', Button, class_='is-info', is_='info-button')
WarningButton = NewTag('WarningButton', 'button', Button, class_='is-warning', is_='warning-button')
DangerButton = NewTag('DangerButton', 'button', Button, class_='is-danger', is_='danger-button')
ErrorButton = NewTag('ErrorButton', 'button', Button, class_='is-danger', is_='error-button')
LinkButton = NewTag('LinkButton', 'button', Button, class_='is-link', is_='link-button')

Input = NewTag('Input', 'input', Input)
TextInput = NewTag('Input', 'input', Input, type_='text', class_='input')
CheckBox = NewTag('Input', 'input', Input, type_='checkbox', class_='checkbox')
RadioButton = NewTag('Input', 'input', Input, type_='radio', class_='radio')
Textarea = NewTag('Textarea', 'textarea', Textarea, class_='textarea')

class Select(NestedTag):
    tag = 'span'
    class_ = 'select'
    inner_tag_class = Select
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._inner_element.addEventListener('change', self.dispatchEvent)
        self._inner_element.addEventListener('click', self.dispatchEvent)

Table = NewTag('Table', 'table', Table, class_='table')

H1 = NewTag('H1', 'h1', H1, class_='title is-1')
H2 = NewTag('H2', 'h2', H2, class_='title is-2')
H3 = NewTag('H3', 'h3', H3, class_='title is-3')
H4 = NewTag('H4', 'h4', H4, class_='title is-4')
H5 = NewTag('H5', 'h5', H5, class_='title is-5')
H6 = NewTag('H6', 'h6', H6, class_='title is-6')

Container = NewTag('Container', 'div', Container, class_='content')
Wrapper = NewTag('Wrapper', 'div', Wrapper, class_='content')
Row = NewTag('Row', 'div', Row, class_='columns')

Col = NewTag('Col', 'div', Col, class_='column')
Col1 = NewTag('Col1', 'div', Col, class_='is-1', is_='col1')
Col2 = NewTag('Col2', 'div', Col, class_='is-2', is_='col2')
Col3 = NewTag('Col3', 'div', Col, class_='is-3', is_='col3')
Col4 = NewTag('Col4', 'div', Col, class_='is-4', is_='col4')
Col5 = NewTag('Col5', 'div', Col, class_='is-5', is_='col5')
Col6 = NewTag('Col6', 'div', Col, class_='is-6', is_='col6')
Col7 = NewTag('Col7', 'div', Col, class_='is-7', is_='col7')
Col8 = NewTag('Col8', 'div', Col, class_='is-8', is_='col8')
Col9 = NewTag('Col9', 'div', Col, class_='is-9', is_='col9')
Col10 = NewTag('Col10', 'div', Col, class_='is-10', is_='col10')
Col11 = NewTag('Col11', 'div', Col, class_='is-11', is_='col11')
Col12 = NewTag('Col12', 'div', Col, class_='is-12', is_='col12')

extended_classes = [
    Button,
    DefaultButton,
    PrimaryButton,
    SecondaryButton,
    SuccessButton,
    InfoButton,
    WarningButton,
    DangerButton,
    ErrorButton,
    LinkButton,
    Input,
    TextInput,
    CheckBox,
    RadioButton,
    Textarea,
    Select,
    Table,
    H1,
    H2,
    H3,
    H4,
    H5,
    H6,
    Container,
    Wrapper,
    Row,
    Col,
    Col1,
    Col2,
    Col3,
    Col4,
    Col5,
    Col6,
    Col7,
    Col8,
    Col9,
    Col10,
    Col11,
    Col12,
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
import re
from collections import OrderedDict
import logging
from typing import Optional

from wdom.interface import Node
from wdom.webif import WebIF

logger = logging.getLogger(__name__)
_css_norm_re = re.compile(r'([a-z])([A-Z])')
_style_cleanup_re = re.compile(r'\s*([:;])\s*', re.S)
_style_rule_re = re.compile(r'\s*(.*?)\s*{(.*?)}\s*', re.S)

if sys.version_info >= (3, 5):
    _dict = OrderedDict
else:
    _dict = dict


def _lower_dash(m):
    return m.group(1) + '-' + m.group(2).lower()


def _normalize_css_property(prop):
    if prop == 'cssFloat':  # Special case
        return 'float'
    else:
        return _css_norm_re.sub(_lower_dash, prop)


class CSSStyleDeclaration(_dict):
    def __init__(self, style: str = None, parent: Optional['CSSRule'] = None,
                 owner: Optional['Node'] = None):
        self.parentRule = parent
        self._owner = owner
        if style:
            self._parse_str(style)

    def _update(self):
        if isinstance(self._owner, WebIF):
            css = self.cssText
            if css:
                self._owner.js_exec('setAttribute', 'style', css)
            else:
                self._owner.js_exec('removeAttribute', 'style')

    def _parse_str(self, style: str) -> None:
        self.clear()
        orig_style = style
        style_str = _style_cleanup_re.sub(r'\1', style.strip())
        if len(style_str) == 0:
            # do nothing, just clear and update
            self._update()
            return

        # temporary disable udpating browser for better performance
        self._owner, _owner = None, self._owner
        for decls in style_str.split(';'):
            if ':' not in decls:
                if len(decls) > 0:
                    logger.warning('[skip] unknown style: {}'.format(decls))
                continue
            decl_list = decls.split(':')
            if len(decl_list) != 2:
                raise ValueError(
                    'Invalid style declaration: {0} in {1}'.format(
                        decls, orig_style))
            prop = decl_list[0]
            value = decl_list[1]
            self[prop] = value
        self._owner = _owner
        self._update()

    @property
    def cssText(self) -> str:
        text = '; '.join('{0}: {1}'.format(k, v) for k, v in self.items())
        if text:
            text += ';'
        return text

    @property
    def length(self) -> int:
        return len(self)

    @property
    def parentRule(self):
        return self._parent

    @parentRule.setter
    def parentRule(self, parent):
        self._parent = parent

    def getPropertyValue(self, prop: str) -> str:
        return self.get(prop, '')

    def removeProperty(self, prop: str) -> str:
        removed_prop = self.get(prop)
        # removed_prop may be False or '', so need to check it is None
        if removed_prop is not None:
            del self[prop]
        return removed_prop

    def setProperty(self, prop: str, value: str, priority=None):
        self[prop] = value

    def __getitem__(self, attr) -> str:
        return self.get(_normalize_css_property(attr), '')

    def __setitem__(self, attr, value) -> str:
        super().__setitem__(_normalize_css_property(attr), value)
        self._update()

    def __delitem__(self, attr):
        super().__delitem__(_normalize_css_property(attr))
        self._update()

    def __getattr__(self, attr: str) -> str:
        if attr.startswith('_') or attr in dir(self):
            return super().__getattr__(attr)
        else:
            return self.get(_normalize_css_property(attr), '')

    def __setattr__(self, attr, value):
        if attr.startswith('_') or attr in dir(self):
            super().__setattr__(attr, value)
        else:
            self[_normalize_css_property(attr)] = value

    def __delattr__(self, attr):
        if attr.startswith('_') or attr in dir(self):
            super().__delattr__(attr)
        else:
            self.__delitem__(_normalize_css_property(attr))


def parse_style_decl(style: str, owner: Node = None) -> CSSStyleDeclaration:
    style = CSSStyleDeclaration(style, owner=owner)
    return style


class CSSStyleRule(object):
    def __init__(self, selector: str = '', style: CSSStyleDeclaration = None):
        self.selectorText = selector
        if style is None:
            self.style = CSSStyleDeclaration()
        else:
            self.style = style

    @property
    def cssText(self) -> str:
        _style = self.style.cssText
        if _style:
            return '{0} {{{1}}}'.format(self.selectorText, _style)
        else:
            return ''


class CSSRuleList(list):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @property
    def length(self) -> int:
        return len(self)

    def item(self, index: int) -> CSSStyleRule:
        return self[index]

    @property
    def cssText(self) -> str:
        return '\n'.join(rule.cssText for rule in self)


def parse_style_rules(styles: str) -> CSSRuleList:
    rules = CSSRuleList()
    for m in _style_rule_re.finditer(styles):
        rules.append(CSSStyleRule(m.group(1), parse_style_decl(m.group(2))))
    return rules



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from wdom.tag import NewTagClass as NewTag
from wdom.tag import *

name = 'Papier'
project_url = 'http://gugel.io/papier/'
project_repository = 'https://github.com/alexanderGugel/papier'
license = 'MIT License'
license_url = 'https://github.com/alexanderGugel/papier/blob/master/LICENSE.md'

css_files = [
    '//cdn.rawgit.com/alexanderGugel/papier/master/dist/papier-1.0.0.min.css',
]

DefaultButton = NewTag('DefaultButton', 'button', Button, class_='bg-white')
PrimaryButton = NewTag('PrimaryButton', 'button', Button, class_='bg-blue')
SecondaryButton = NewTag('SecondaryButton', 'button', Button, class_='bg-grey')
SuccessButton = NewTag('SuccessButton', 'button', Button, class_='bg-light-blue')
InfoButton = NewTag('InfoButton', 'button', Button, class_='bg-green')
WarningButton = NewTag('WarningButton', 'button', Button, class_='bg-orange')
DangerButton = NewTag('DangerButton', 'button', Button, class_='bg-red')
ErrorButton = NewTag('ErrorButton', 'button', Button, class_='bg-red')
LinkButton = NewTag('LinkButton', 'button', Button, class_='bg-cyan')

Row = NewTag('Row', 'div', Row, class_='row')
Col1 = NewTag('Col1', 'div', Col1, class_='col-1')
Col2 = NewTag('Col2', 'div', Col2, class_='col-2')
Col3 = NewTag('Col3', 'div', Col3, class_='col-3')
Col4 = NewTag('Col4', 'div', Col4, class_='col-4')
Col5 = NewTag('Col5', 'div', Col5, class_='col-5')
Col6 = NewTag('Col6', 'div', Col6, class_='col-6')
Col7 = NewTag('Col7', 'div', Col7, class_='col-7')
Col8 = NewTag('Col8', 'div', Col8, class_='col-8')
Col9 = NewTag('Col9', 'div', Col9, class_='col-9')
Col10 = NewTag('Col10', 'div', Col10, class_='col-10')
Col11 = NewTag('Col11', 'div', Col11, class_='col-11')
Col12 = NewTag('Col12', 'div', Col12, class_='col-12')

extended_classes = [
    DefaultButton,
    PrimaryButton,
    SecondaryButton,
    SuccessButton,
    InfoButton,
    WarningButton,
    DangerButton,
    ErrorButton,
    LinkButton,
    Row,
    Col1,
    Col2,
    Col3,
    Col4,
    Col5,
    Col6,
    Col7,
    Col8,
    Col9,
    Col10,
    Col11,
    Col12,
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
from os import path

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

readme_file = path.join(path.dirname(path.abspath(__file__)), 'README.rst')
with open(readme_file) as readme_file:
    readme = readme_file.read()

install_requires = ['tornado']
test_requites = ['nose_parameterized', 'selenium', 'syncer']

if sys.version_info < (3, 5):
    install_requires.append('mypy-lang')


setup(
    name='wdom',
    version='0.1.4',
    description='GUI library for browser-based desktop applications',
    long_description=readme,
    author='Hiroyuki Takagi',
    author_email='miyako.dev@gmail.com',
    url='https://github.com/miyakogi/wdom',
    packages=[
        'wdom',
        'wdom.examples',
        'wdom.server',
        'wdom.themes',
        'wdom.tests',
        'wdom.tests.local_browser',
        'wdom.tests.remote_browser',
    ],
    include_package_data=True,
    license="MIT",
    zip_safe=False,
    keywords='dom browser',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='wdom.tests',

    install_requires=install_requires,
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
from os import path
import subprocess

from nose_parameterized import parameterized

from wdom.testing import TestCase

root = path.dirname(path.dirname(path.dirname(path.abspath(__file__))))

cases = [
    ('wdom', 'css'),
    ('wdom', 'document'),
    ('wdom', 'element'),
    ('wdom', 'event'),
    ('wdom', 'interface'),
    ('wdom', 'misc'),
    ('wdom', 'node'),
    ('wdom', 'options'),
    ('wdom', 'parser'),
    ('wdom', 'tag'),
    ('wdom', 'testing'),
    ('wdom', 'web_node'),
    ('wdom', 'webif'),
    ('wdom', 'window'),
    ('wdom', 'server'),
    ('wdom.server', 'base'),
    ('wdom.server', 'handler'),
    ('wdom.server', '_tornado'),
    ('wdom', 'themes'),
    ('wdom.themes', 'default'),
    ('wdom.themes', 'kube'),
    ('wdom.examples', 'data_binding'),
    ('wdom.examples', 'rev_text'),
    ('wdom.examples', 'theming'),
]
try:
    import aiohttp
    cases.append(('wdom.server', '_aiohttp'))
    del aiohttp
except ImportError:
    pass


class TestImportModules(TestCase):
    @parameterized.expand(cases)
    def test_import(self, from_, import_):
        cmd = 'from {0} import {1}\nlist(vars({1}).items())'
        proc = subprocess.Popen(
            [sys.executable, '-c', cmd.format(from_, import_)],
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            cwd=root,
        )
        proc.wait()
        if proc.returncode != 0:
            print(proc.stdout.read())
        self.assertEqual(proc.returncode, 0)

    def test_wdom_import(self):
        cmd = 'import wdom\nlist(vars(wdom).items())'
        proc = subprocess.Popen(
            [sys.executable, '-c', cmd],
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            cwd=root,
        )
        proc.wait()
        if proc.returncode != 0:
            print(proc.stdout.read())
        self.assertEqual(proc.returncode, 0)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
import re
import copy
import pathlib
import webbrowser
from typing import Optional

from tornado import autoreload

from wdom.options import config


exclude_patterns = [
    r'node_modules',
    r'__pycache__',
    r'\..*',
]
_exclude_patterns_re = []
_exclude_patterns_prev = []


def _compile_exclude_patterns():
    global _exclude_patterns_re, _exclude_patterns_prev
    if _exclude_patterns_prev == exclude_patterns:
        return
    else:
        _exclude_patterns_prev = copy.copy(exclude_patterns)
    for pat in exclude_patterns:
        _exclude_patterns_re.append(re.compile(pat))


def _is_exclude(name: str):
    return any(pat.match(name) for pat in _exclude_patterns_re)


def _add_watch_path(path: pathlib.Path):
    if _is_exclude(path.name):
        return
    elif path.is_dir():
        for f in path.iterdir():
            _add_watch_path(f)
    elif path.is_file():
        autoreload.watch(str(path))


def watch_dir(path: str):
    """Add ``path`` to watch for autoreload."""
    _compile_exclude_patterns()
    if config.autoreload or config.debug:
        # Add files to watch for autoreload
        p = pathlib.Path(path)
        p.resolve()
        _add_watch_path(p)


def open_browser(url, browser: Optional[str] = None):
    """Open web browser."""
    if '--open-browser' in sys.argv:
        # Remove open browser to prevent making new tab on autoreload
        sys.argv.remove('--open-browser')
    if browser is None:
        browser = config.browser
    if browser in webbrowser._browsers:
        webbrowser.get(browser).open(url)
    else:
        webbrowser.open(url)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from wdom.tag import NewTagClass as NewTag
from wdom.tag import *

name = 'Skeleton'
project_url = 'http://getskeleton.com/'
project_repository = 'https://github.com/dhg/Skeleton'
license = 'MIT License'
license_url = 'https://github.com/dhg/Skeleton/blob/master/LICENSE.md'

css_files = [
    '//cdnjs.cloudflare.com/ajax/libs/skeleton/2.0.4/skeleton.min.css',
]

PrimaryButton = NewTag('PrimaryButton', 'button', Button, class_='button-primary', is_='primary-button')
Container = NewTag('Container', 'div', Container, class_='container')
Wrapper = NewTag('Wrapper', 'div', Wrapper, class_='container')
Row = NewTag('Row', 'div', Row, class_='row')
Col = NewTag('Col', 'div', Col, class_='columns')
Col1 = NewTag('Col1', 'div', Col1, class_='one column')
Col2 = NewTag('Col2', 'div', Col2, class_='two columns')
Col3 = NewTag('Col3', 'div', Col3, class_='three columns')
Col4 = NewTag('Col4', 'div', Col4, class_='four columns')
Col5 = NewTag('Col5', 'div', Col5, class_='five columns')
Col6 = NewTag('Col6', 'div', Col6, class_='six columns')
Col7 = NewTag('Col7', 'div', Col7, class_='seven columns')
Col8 = NewTag('Col8', 'div', Col8, class_='eight columns')
Col9 = NewTag('Col9', 'div', Col9, class_='nine columns')
Col10 = NewTag('Col10', 'div', Col10, class_='ten columns')
Col11 = NewTag('Col11', 'div', Col11, class_='eleven columns')
Col12 = NewTag('Col12', 'div', Col12, class_='twelve columns')

extended_classes = [
    PrimaryButton,
    Container,
    Wrapper,
    Row,
    Col,
    Col1,
    Col2,
    Col3,
    Col4,
    Col5,
    Col6,
    Col7,
    Col8,
    Col9,
    Col10,
    Col11,
    Col12,
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from wdom.tag import NewTagClass as NewTag
from wdom.tag import *

name = 'Baseguide'
project_url = 'http://basegui.de/'
project_repository = 'https://github.com/slavanga/baseguide'
license = 'MIT License'
license_url = 'https://github.com/slavanga/baseguide/blob/master/LICENSE'

css_files = [
    '_static/css/baseguide.min.css',
]

Button = NewTag('Button', bases=Button, class_='btn')
DefaultButton = NewTag('DefaultButton', 'button', Button, class_='btn-default', is_='default-button')
PrimaryButton = NewTag('PrimaryButton', 'button', Button, class_='btn-primary', is_='primary-button')
SecondaryButton = NewTag('SecondaryButton', 'button', Button, class_='btn-secondary', is_='secondary-button')
SuccessButton = NewTag('SuccessButton', 'button', Button, class_='btn-success', is_='success-button')
InfoButton = NewTag('InfoButton', 'button', Button, class_='btn-info', is_='info-button')
WarningButton = NewTag('WarningButton', 'button', Button, class_='btn-warning', is_='warning-button')
DangerButton = NewTag('DangerButton', 'button', Button, class_='btn-danger', is_='danger-button')
ErrorButton = NewTag('ErrorButton', 'button', Button, class_='btn-danger', is_='error-button')
LinkButton = NewTag('LinkButton', 'button', Button, class_='btn-link', is_='link-button')

FormGroup = NewTag('FormGroup', 'div', FormGroup, class_='form-group')
Input = NewTag('Input', 'input', Input, class_='form-control')
TextInput = NewTag('TextInput', 'input', TextInput, class_='form-control')
Textarea = NewTag('Textarea', 'textarea', Textarea, class_='form-control')
Select = NewTag('Select', 'select', Select, class_='form-control')

Table = NewTag('Table', 'table', Table, class_='table')

Container = NewTag('Container', 'div', Container, class_='container')
Wrapper = NewTag('Wrapper', 'div', Wrapper, class_='container')
Row = NewTag('Row', 'div', Row, class_='row')

Col = NewTag('Col', 'div', Col, class_='col')
Col1 = NewTag('Col1', 'div', Col1, class_='col col-xs-1 col-sm-1 col-md-1 col-lg-1')
Col2 = NewTag('Col2', 'div', Col2, class_='col col-xs-2 col-sm-2 col-md-2 col-lg-2')
Col3 = NewTag('Col3', 'div', Col3, class_='col col-xs-3 col-sm-3 col-md-3 col-lg-3')
Col4 = NewTag('Col4', 'div', Col4, class_='col col-xs-4 col-sm-4 col-md-4 col-lg-4')
Col5 = NewTag('Col5', 'div', Col5, class_='col col-xs-5 col-sm-5 col-md-5 col-lg-5')
Col6 = NewTag('Col6', 'div', Col6, class_='col col-xs-6 col-sm-6 col-md-6 col-lg-6')
Col7 = NewTag('Col7', 'div', Col7, class_='col col-xs-7 col-sm-7 col-md-7 col-lg-7')
Col8 = NewTag('Col8', 'div', Col8, class_='col col-xs-8 col-sm-8 col-md-8 col-lg-8')
Col9 = NewTag('Col9', 'div', Col9, class_='col col-xs-9 col-sm-9 col-md-9 col-lg-9')
Col10 = NewTag('Col10', 'div', Col10, class_='col col-xs-10 col-sm-10 col-md-10 col-lg-10')
Col11 = NewTag('Col11', 'div', Col11, class_='col col-xs-11 col-sm-11 col-md-11 col-lg-11')
Col12 = NewTag('Col12', 'div', Col12, class_='col col-xs-12 col-sm-12 col-md-12 col-lg-12')

extended_classes = [
    Button,
    DefaultButton,
    PrimaryButton,
    SecondaryButton,
    SuccessButton,
    InfoButton,
    WarningButton,
    DangerButton,
    ErrorButton,
    LinkButton,
    FormGroup,
    Input,
    TextInput,
    Textarea,
    Select,
    Table,
    Container,
    Wrapper,
    Row,
    Col,
    Col1,
    Col2,
    Col3,
    Col4,
    Col5,
    Col6,
    Col7,
    Col8,
    Col9,
    Col10,
    Col11,
    Col12,
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
import unittest

from nose_parameterized import parameterized

from wdom.css import _normalize_css_property
from wdom.css import CSSStyleDeclaration, parse_style_decl
from wdom.css import CSSStyleRule, parse_style_rules
from wdom.css import CSSRuleList
from wdom.testing import TestCase


class TestCSSProperties(TestCase):
    @parameterized.expand([
        ('color', 'color'),
        ('fontColor', 'font-color'),
        ('borderWidth', 'border-width'),
        ('borderBottomColor', 'border-bottom-color'),
        ('zIndex', 'z-index'),
        ('cssFloat', 'float'),
    ])
    def test_normalize(self, js, css):
        self.assertEqual(_normalize_css_property(js), css)


class TestCSSStyleDeclaration(TestCase):
    def setUp(self):
        self.css = CSSStyleDeclaration()

    def test_init(self):
        self.assertEqual(self.css.cssText, '')
        self.assertEqual(self.css.length, 0)

    def test_set_get_remove(self):
        self.assertEqual(self.css.getPropertyValue('color'), '')

        self.css.setProperty('color', 'red')
        self.assertEqual(self.css.getPropertyValue('color'), 'red')
        self.assertEqual(self.css['color'], 'red')
        self.assertEqual(self.css.cssText, 'color: red;')
        self.assertEqual(self.css.length, 1)

        self.css.setProperty('font-color', '#000')
        self.assertEqual(self.css.getPropertyValue('color'), 'red')
        self.assertEqual(self.css['color'], 'red')
        self.assertEqual(self.css.getPropertyValue('font-color'), '#000')
        self.assertEqual(self.css['font-color'], '#000')
        self.assertIn('font-color: #000;', self.css.cssText)
        self.assertIn('color: red;', self.css.cssText)
        self.assertEqual(self.css.length, 2)

        # removedProperty returns value of removed property
        self.assertEqual(self.css.removeProperty('color'), 'red')

        self.assertEqual(self.css.getPropertyValue('color'), '')
        self.assertEqual(self.css['color'], '')
        self.assertEqual(self.css.getPropertyValue('font-color'), '#000')
        self.assertEqual(self.css['font-color'], '#000')
        self.assertEqual(self.css.cssText, 'font-color: #000;')
        self.assertNotIn('color: red;', self.css.cssText)
        self.assertEqual(self.css.length, 1)

    def test_property_access(self):
        self.assertEqual(self.css.color, '')
        self.css.color = 'red'
        self.assertEqual(self.css.color, 'red')
        self.assertEqual(self.css.getPropertyValue('color'), 'red')
        self.assertEqual(self.css['color'], 'red')
        self.assertEqual(self.css.cssText, 'color: red;')
        self.assertEqual(self.css.length, 1)

        del self.css.color
        self.assertEqual(self.css.color, '')
        self.assertEqual(self.css.getPropertyValue('color'), '')
        self.assertEqual(self.css.cssText, '')
        self.assertEqual(self.css.length, 0)

    def test_property_access_dash(self):
        self.assertEqual(self.css.zIndex, '')
        self.assertEqual(self.css.getPropertyValue('z-index'), '')
        self.assertEqual(self.css.getPropertyValue('zIndex'), '')

        self.css.zIndex = 1
        self.assertEqual(self.css.zIndex, 1)
        self.assertEqual(self.css.getPropertyValue('z-index'), 1)
        self.assertEqual(self.css.getPropertyValue('zIndex'), '')
        self.assertEqual(self.css.cssText, 'z-index: 1;')
        self.assertEqual(self.css.length, 1)

        self.css.setProperty('z-index', 2)
        self.assertEqual(self.css.zIndex, 2)
        self.assertEqual(self.css.getPropertyValue('z-index'), 2)
        self.assertEqual(self.css.getPropertyValue('zIndex'), '')
        self.assertEqual(self.css.cssText, 'z-index: 2;')
        self.assertEqual(self.css.length, 1)

        del self.css.zIndex
        self.assertEqual(self.css.zIndex, '')
        self.assertEqual(self.css.getPropertyValue('z-index'), '')
        self.assertEqual(self.css.getPropertyValue('zIndex'), '')
        self.assertEqual(self.css.cssText, '')
        self.assertEqual(self.css.length, 0)

    def test_property_access_dash_two(self):
        self.assertEqual(self.css.listStyleType, '')
        self.assertEqual(self.css.getPropertyValue('list-style-type'), '')
        self.assertEqual(self.css.getPropertyValue('listStyleType'), '')

        self.css.listStyleType = 'disc'
        self.assertEqual(self.css.listStyleType, 'disc')
        self.assertEqual(self.css.getPropertyValue('list-style-type'), 'disc')
        self.assertEqual(self.css.getPropertyValue('listStyleType'), '')
        self.assertEqual(self.css.cssText, 'list-style-type: disc;')
        self.assertEqual(self.css.length, 1)

        del self.css.listStyleType
        self.assertEqual(self.css.listStyleType, '')
        self.assertEqual(self.css.getPropertyValue('list-style-type'), '')
        self.assertEqual(self.css.getPropertyValue('listStyleType'), '')
        self.assertEqual(self.css.cssText, '')
        self.assertEqual(self.css.length, 0)

    def test_float(self):
        self.assertEqual(self.css.cssFloat, '')
        self.assertEqual(self.css.getPropertyValue('float'), '')
        self.assertEqual(self.css.getPropertyValue('cssFloat'), '')
        self.assertEqual(self.css.getPropertyValue('css-float'), '')

        self.css.cssFloat = 1
        self.assertEqual(self.css.getPropertyValue('cssFloat'), '')
        self.assertEqual(self.css.getPropertyValue('float'), 1)
        self.assertEqual(self.css.getPropertyValue('css-float'), '')


class TestCSSParseDecl(TestCase):
    @parameterized.expand([
        ('color:red;', 'color: red;'),
        ('color:red', 'color: red;'),
        ('color  :red  ;', 'color: red;'),
        ('margin: 1 3 4 5;', 'margin: 1 3 4 5;'),
        (' margin :1 3 4 5   ;  ', 'margin: 1 3 4 5;'),
        ('z-index: 1;', 'z-index: 1;'),
        ('color: red; z-index: 1;', 'color: red; z-index: 1;'),
        ('  color  : red ;  z-index  : 1  ', 'color: red; z-index: 1;'),
        ('color: red;  \n z-index: 1;', 'color: red; z-index: 1;'),
    ])
    @unittest.skipIf(sys.version_info < (3, 5), 'py34 can\'t keep order')
    def test_parse_style_order(self, input, css):
        self.assertEqual(parse_style_decl(input).cssText, css)

    @parameterized.expand([
        ('color:red;', 'color: red;'),
        ('color:red', 'color: red;'),
        ('color  :red  ;', 'color: red;'),
        ('margin: 1 3 4 5;', 'margin: 1 3 4 5;'),
        (' margin :1 3 4 5   ;  ', 'margin: 1 3 4 5;'),
        ('z-index: 1;', 'z-index: 1;'),
        ('color: red; z-index: 1;', 'color: red;', 'z-index: 1;'),
        ('  color  : red ;  z-index  : 1  ', 'color: red;', 'z-index: 1;'),
        ('color: red;  \n z-index: 1;', 'color: red;', 'z-index: 1;'),
    ])
    def test_parse_style_unordered(self, input, *csses):
        parsed_css = parse_style_decl(input).cssText
        for css in csses:
            self.assertIn(css, parsed_css)


class TestCSSStyleRule(TestCase):
    def setUp(self):
        self.rule = CSSStyleRule()

    def test_blank(self):
        self.assertEqual(self.rule.cssText, '')
        self.rule.selectorText = 'h1'
        self.assertEqual(self.rule.cssText, '')

    def test_init(self):
        style = CSSStyleDeclaration()
        style.color = 'red'
        rule = CSSStyleRule('h1', style)
        self.assertEqual(rule.cssText, 'h1 {color: red;}')

    def test_overwrite_style(self):
        self.rule.style = CSSStyleDeclaration()
        self.rule.style.color = 'red'
        self.assertEqual(self.rule.cssText, ' {color: red;}')

        self.rule.selectorText = 'h1'
        self.assertEqual(self.rule.cssText, 'h1 {color: red;}')

        self.rule.selectorText = 'h1,h2'
        self.assertEqual(self.rule.cssText, 'h1,h2 {color: red;}')

        self.rule.style.removeProperty('color')
        self.assertEqual(self.rule.cssText, '')


class TestCSSRuleList(TestCase):
    def setUp(self):
        self.list = CSSRuleList()
        self.style = CSSStyleDeclaration()
        self.style.color = 'red'
        self.rule = CSSStyleRule('h1', self.style)

    def test_blank(self):
        self.assertEqual(self.list.cssText, '')

    def test_append(self):
        self.list.append(self.rule)
        self.assertEqual(self.list.cssText, 'h1 {color: red;}')

    def test_append2(self):
        self.list.append(self.rule)
        rule2 = CSSStyleRule('h2', CSSStyleDeclaration('background: black;'))
        self.list.append(rule2)
        css = self.list.cssText
        if sys.version_info < (3, 5):
            # python 3.4 can't keep order
            self.assertIn('h1 {color: red;}', css)
            self.assertIn('h2 {background: black;}', css)
        else:
            self.assertIn('h1 {color: red;}\nh2 {background: black;}', css)


class TestParseRules(TestCase):
    @parameterized.expand([
        ('h1 {color:red;}', 'h1 {color: red;}'),
        ('h1,h2 {color:red;}', 'h1,h2 {color: red;}'),
        ('  h1  {  color  :  red  ;  }', 'h1 {color: red;}'),
        ('h1{color:red}', 'h1 {color: red;}'),
        ('h1 {color: red;}\n   h2 {font-size: 4px;}',
         'h1 {color: red;}\nh2 {font-size: 4px;}'),
        ('h1 {\n  color: red;\n  background: white;}\n h2 {font-size: 4px;}',
         'h1 {color: red; background: white;}\nh2 {font-size: 4px;}'),
    ])
    @unittest.skipIf(sys.version_info < (3, 5), 'python3.4 can\'t keep order')
    def test_parse_style_rules(self, input, rule):
        self.assertEqual(rule, parse_style_rules(input).cssText)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from wdom.tag import NewTagClass as NewTag
from wdom.tag import *

name = 'Bijou'
project_url = 'http://andhart.github.io/bijou/'
project_repository = 'https://github.com/andhart/bijou'
license = 'MIT License'
license_url = 'https://github.com/andhart/bijou/blob/master/LICENSE'

css_files = [
    '_static/css/bijou.min.css',
]

Button = NewTag('Button', bases=Button, class_='button small')
DefaultButton = NewTag('DefaultButton', 'button', Button, is_='default-button')
PrimaryButton = NewTag('PrimaryButton', 'button', Button, class_='primary', is_='primary-button')
SecondaryButton = NewTag('SecondaryButton', 'button', Button, is_='secondary-button')
SuccessButton = NewTag('SuccessButton', 'button', Button, class_='success', is_='success-button')
InfoButton = NewTag('InfoButton', 'button', Button, class_='success', is_='info-button')
WarningButton = NewTag('WarningButton', 'button', Button, class_='danger', is_='warning-button')
DangerButton = NewTag('DangerButton', 'button', Button, class_='danger', is_='danger-button')
ErrorButton = NewTag('ErrorButton', 'button', Button, class_='danger', is_='error-button')
LinkButton = NewTag('LinkButton', 'button', Button, is_='link-button')

Table = NewTag('Table', 'table', Table, class_='table')

Row = NewTag('Row', 'div', Row, class_='row')
Col = NewTag('Col', 'div', Col, class_='span')
Col1 = NewTag('Col1', 'div', (Col1, Col), class_='one')
Col2 = NewTag('Col2', 'div', (Col2, Col), class_='two')
Col3 = NewTag('Col3', 'div', (Col3, Col), class_='three')
Col4 = NewTag('Col4', 'div', (Col4, Col), class_='four')
Col5 = NewTag('Col5', 'div', (Col5, Col), class_='five')
Col6 = NewTag('Col6', 'div', (Col6, Col), class_='six')
Col7 = NewTag('Col7', 'div', (Col7, Col), class_='seven')
Col8 = NewTag('Col8', 'div', (Col8, Col), class_='eight')
Col9 = NewTag('Col9', 'div', (Col9, Col), class_='nine')
Col10 = NewTag('Col10', 'div', (Col10, Col), class_='ten')
Col11 = NewTag('Col11', 'div', (Col11, Col), class_='eleven')

extended_classes = [
    Button,
    DefaultButton,
    PrimaryButton,
    SuccessButton,
    InfoButton,
    WarningButton,
    DangerButton,
    ErrorButton,
    LinkButton,
    Table,
    Row,
    Col1,
    Col2,
    Col3,
    Col4,
    Col5,
    Col6,
    Col7,
    Col8,
    Col9,
    Col10,
    Col11,
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import sys
from pathlib import Path

CURFILE = Path(__file__).resolve()
CURDIR = CURFILE.parent.resolve()

if __name__ == '__main__':
    sys.path.insert(0, str(CURDIR.parent.parent.resolve()))

from wdom import options, tag
from wdom.themes import default
from wdom.document import Document, get_document


def _get_theme_name(theme) -> str:
    if hasattr(theme, 'name'):
        theme_name = theme.name
    elif theme.__name__ == 'wdom.themes':
        if options.config.theme:
            theme_name = options.config.theme.upper()
        else:
            theme_name = 'DEFAULT'
    return theme_name


def sample_app(theme=default) -> tag.Tag:
    app = theme.Div()
    app['style'] = '''\
        margin: 2em;
    '''
    body = theme.Container(parent=app)
    body.append(theme.Div(theme.H1(_get_theme_name(theme)),
                          style='text-align: center;'))
    body.append(theme.Hr())

    button_wrapper = theme.Div(parent=body)
    button_wrapper.append(
        theme.Button('Button'),
        theme.DefaultButton('Default'),
        theme.PrimaryButton('Primary'),
        theme.SecondaryButton('Secondary'),
        theme.SuccessButton('Success'),
        theme.InfoButton('Info'),
        theme.WarningButton('Warning'),
        theme.DangerButton('Danger'),
        theme.ErrorButton('Error'),
        theme.LinkButton('Link'),
    )
    body.append(theme.Hr())

    input_wrapper = theme.Form(parent=theme.FormGroup(parent=body))
    input_wrapper.append(
        theme.Textarea(placeholder='<textarea></textarea>'),
        theme.Input(placeholder='<input type="text">'),
        theme.Label(theme.CheckBox(id='checkbox1'), 'CheckBox 1'),
        theme.CheckBox(id='checkbox2'),
        theme.Label('CheckBox 2', **{'for':'checkbox2'}),
        theme.RadioButton(id='radio1', name='radio_group'),
        theme.Label('RadioButton 1', **{'for':'radio1'}),
        theme.RadioButton(id='radio2', name='radio_group'),
        theme.Label('RadioButton 2', **{'for':'radio2'}),
    )

    dropdown_list = theme.Select(parent=body)
    dropdown_list.append(
        theme.Option('Item 1'),
        theme.Optgroup(
            theme.Option('Item 2'),
            theme.Option('Item 3'),
            label='- Item Group -'
        )
    )
    multi_select = theme.Select(parent=body, multiple=True)
    multi_select.append(
        theme.Option('Option 1'),
        theme.Optgroup(
            theme.Option('Option 2'),
            theme.Option('Option 3'),
            label='- Option Group -'
        )
    )
    body.append(theme.Hr())

    # List
    list_div = theme.Row(parent=body)
    ul_div = theme.Col6(parent=list_div)
    ul_div.append(theme.H3('Unordered List'))
    ul1 = theme.Ul(parent=ul_div)
    ul1.append(theme.Li('Item 1'))
    li = theme.Li('Item 2', parent=ul1)
    ul2 = theme.Ul(parent=li)
    ul2.append(
        theme.Li('Item 2.1'),
        theme.Li('Item 2.2'),
        theme.Li('Item 3'),
    )

    ol_div = theme.Col6(parent=list_div)
    ol_div.append(theme.H3('Ordered List'))
    ol1 = theme.Ol(parent=ol_div)
    ol1.append(theme.Li('Item 1'))
    li = theme.Li('Item 2', parent=ol1)
    ol2 = theme.Ol(parent=li)
    ol2.append(
        theme.Li('Item 2.1'),
        theme.Li('Item 2.2'),
        theme.Li('Item 3'),
    )
    body.append(theme.Hr())

    code_div = theme.Div(parent=body)
    pre = theme.Pre(parent=code_div)
    pre.append(theme.Code(
        '''\
    def python(i: int) -> str:
        print(str(i))
    '''.rstrip()))
    body.append(theme.Hr())

    # Table
    table_div = theme.Div(parent=body)
    table = theme.Table(parent=table_div)

    thead = theme.Thead(parent=table)
    tr1 = theme.Tr(parent=thead)
    tr1.append(
        theme.Th('Name'),
        theme.Th('Age'),
        theme.Th('Seibetsu'),
        theme.Th('Location'),
    )

    tbody = theme.Tbody(parent=table)
    tr2 = theme.Tr(parent=tbody)
    tr2.append(
        theme.Td('Ichiro Suzuki'),
        theme.Td('26'),
        theme.Td('Male'),
        theme.Td('Tokyo'),
    )

    tr3 = theme.Tr(parent=tbody)
    tr3.append(
        theme.Td('Hikari Takiguchi'),
        theme.Td('20'),
        theme.Td('Female'),
        theme.Td('Kyoto'),
    )
    body.append(theme.Hr())

    # Typography
    typography = theme.Div(parent=body)
    typography.append(
        theme.H1('Heading 1'),
        theme.H2('Heading 2'),
        theme.H3('Heading 3'),
        theme.H4('Heading 4'),
        theme.H5('Heading 5'),
        theme.H6('Heading 6'),
        theme.P('This is the base paragraph.'),
        theme.Strong('Bolded'),
        theme.Div(theme.Em('Italicized')),
        theme.Div(theme.A('Colored (link)')),
        theme.Div(theme.U('Underlined')),
    )
    body.append(theme.Hr())

    grid = theme.Container(parent=body,
                           style='border: solid 1px #bbb; background-color: #666;')
    left_style = 'background-color: #eeeeee;'
    right_style = 'background-color: #fafafa'
    for i in range(0, 13):
        l = 'Col' + str(i or '')
        r = 'Col' + str(12 - i or '')
        grid.append(theme.Row(
            getattr(theme, l)(l, style=left_style),
            getattr(theme, r)(r, style=right_style),
        ))

    return app


def sample_page(theme=default) -> Document:
    page = get_document()
    app = sample_app(theme)
    page.body.prepend(app)
    for css in theme.css_files:
        page.add_cssfile(css)
    for js in theme.js_files:
        page.add_jsfile(js)
    return page


def main():
    import asyncio
    from wdom.server import start_server, get_app, stop_server
    doc = sample_page()
    app = get_app(document=doc)
    loop = asyncio.get_event_loop()
    server = start_server(app=app, loop=loop)
    try:
        loop.run_forever()
    except KeyboardInterrupt:
        stop_server(server)
        loop.close()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from unittest import TestCase

from wdom.interface import Event, NodeList


class TestEvent(TestCase):
    def setUp(self):
        self.e = Event('event')

    def test_event_type(self):
        self.assertEqual(self.e.type, 'event')


class TestNodeList(TestCase):
    def setUp(self):
        self.nl = NodeList(list(range(3)))

    def test_length(self):
        self.assertEqual(self.nl.length, 3)
        self.assertEqual(len(self.nl), 3)

    def test_index_access(self):
        self.assertEqual(self.nl[1], 1)
        self.assertEqual(self.nl[-1], 2)
        self.assertEqual(self.nl[1:2], [1])
        with self.assertRaises(IndexError):
            self.nl[5]

    def test_item(self):
        self.assertEqual(self.nl.item(1), 1)
        self.assertEqual(self.nl.item(-1), None)
        self.assertEqual(self.nl.item(5), None)
        with self.assertRaises(TypeError):
            self.nl.item(slice(1, 2))

    def test_contains(self):
        self.assertTrue(1 in self.nl)
        self.assertFalse(5 in self.nl)

    def test_iteration(self):
        l = [0, 1, 2]
        for n in self.nl:
            self.assertEqual(n, l.pop(0))
        l = [2, 1, 0]
        for n in reversed(self.nl):
            self.assertEqual(n, l.pop(0))

    def test_index(self):
        self.assertEqual(self.nl.index(0), 0)
        self.assertEqual(self.nl.index(1), 1)
        self.assertEqual(self.nl.index(2), 2)
